@@ -801,6 +801,13 @@ field.Adapter = declare(null, {
      */
     result_index: 0,
 
+    /**
+     * Name of the record which we want to extract from the result.
+     * Used in dnslocations.
+     * @type {String}
+     */
+    result_name: 'result',
+
     /**
      * Extract record from RPC call response
      *
@@ -821,10 +828,10 @@ field.Adapter = declare(null, {
         var dr = data.result;
         var record = null;
         if (dr) {
-            if (IPA.defined(dr.result)) record = dr.result;
+            if (IPA.defined(dr[this.result_name])) record = dr[this.result_name];
             else if (dr.results) {
                 var result = dr.results[this.result_index];
-                if (result) record = result.result;
+                if (result) record = result[this.result_name];
             }
         }
         return record;
@@ -1197,6 +1204,61 @@ field.sshkeys_field = IPA.sshkeys_field = function(spec) {
     return that;
 };
 
+
+/**
+ * Field for certificates widget.
+ * - has the ObjectAdapter as default
+ * - by default has  `w_if_no_aci` to workaround missing object class
+ *
+ * @class
+ * @alternateClassName IPA.certs_field
+ * @extends IPA.field
+ */
+field.certs_field = IPA.certs_field = function(spec) {
+    spec = spec || {};
+    spec.adapter = spec.adapter || field.ObjectAdapter;
+    spec.flags = spec.flags || ['w_if_no_aci'];
+
+    var that = IPA.field(spec);
+
+    /**
+     * The index of record from batch command where ACLs are returned.
+     * Necessary for correct display 'add' and 'delete' buttons in certificate
+     * widget.
+     *
+     * @param {Number} acl_result_index
+     */
+    that.acl_result_index = spec.acl_result_index;
+
+    that.load = function(data) {
+        var value = that.adapter.load(data);
+        var parsed = util.parse(that.data_parser, value, "Parse error:"+that.name);
+        value = parsed.value;
+        if (!parsed.ok) {
+            window.console.warn(parsed.message);
+        }
+
+        // specific part for certificates - it is necessary to read rights from
+        // result of user-show command not from cert-find result.
+        // Therefore we need to get record with different index. The correct
+        // index is set in acl_result_index variable, old index is stored
+        // and then put back.
+        var old_index = that.adapter.result_index;
+        if (that.acl_result_index !== undefined) {
+            that.adapter.result_index = that.acl_result_index;
+        }
+
+        var record = that.adapter.get_record(data);
+        that.adapter.result_index = old_index;
+
+        that.load_writable(record);
+
+        that.set_value(value, true);
+    };
+
+    return that;
+};
+
 /**
  * SSH Keys Adapter
  * @class
@@ -1259,6 +1321,85 @@ field.SshKeysAdapter = declare([field.Adapter], {
     }
 });
 
+
+/**
+ * ObjectAdapter is basic adapter which converts object to more usable format.
+ * All properties which have only one value are tranformed this way:
+ *  property1: {"__base64__": "value1"} => property1: "value1",
+ *  property2: {"value2"} => property2: "value2",
+ * Works for __base64__ as well as for __datetime__ and __dns_name__
+ *
+ * In case that the property has more values, then they are returned as array.
+ *
+ * @class
+ * @extends field.Adapter
+ */
+field.ObjectAdapter = declare([field.Adapter], {
+
+    normalize_object: function(obj) {
+        for (var property in obj) {
+            if (obj.hasOwnProperty(property)) {
+                obj[property] = rpc.extract_objects([obj[property]]);
+                if (obj[property].length == 1) {
+                    obj[property] = obj[property][0];
+                }
+            }
+        }
+    },
+
+    load: function(data) {
+
+        var record = this.get_record(data);
+
+        for (var i=0; i<record.length; i++) {
+            this.normalize_object(record[i]);
+        }
+
+        return record;
+    }
+});
+
+
+/**
+ * Custom adapter for fields which handles situations when there is no value
+ * for attribute (name) of the field and we want to use alternative attribute
+ * from response. We can set the alternative attribute name to the 'alt_attr'
+ * attribute of the adapter.
+ * This adapter is used i.e. in table in search facet for services. Handles
+ * situations where older services don't have canonical name.
+ *
+ * @class
+ * @extends field.Adapter
+ */
+field.AlternateAttrFieldAdapter = declare([field.Adapter], {
+    /**
+     * In case that the value is not get using field name then use alternative
+     * name.
+     * @param {Object} data Object which contains the record or the record
+     * @param {string} [attribute] attribute name - overrides `context.param`
+     * @param {Mixed} [def_val] default value - overrides `context.default_value`
+     * @returns {Array} attribute value
+     */
+    load: function(data, attribute, def_val) {
+        var record = this.get_record(data);
+        var value = null;
+        var attr = attribute || this.context.param;
+        var def = def_val || this.context.default_value;
+        if (record) {
+            value = this.get_value(record, attr);
+            if (util.is_empty(value) && this.context.adapter.alt_attr) {
+                value = this.get_value(record, this.context.adapter.alt_attr);
+            }
+        }
+        if (util.is_empty(value) && !util.is_empty(def)) {
+            value = util.normalize_value(def);
+        }
+        value = rpc.extract_objects(value);
+        return value;
+    }
+});
+
+
 /**
  * Field for enabling/disabling entity
  *
@@ -1507,6 +1648,7 @@ field.register = function() {
     var v = reg.validator;
     var l = reg.adapter;
 
+    f.register('certs', field.certs_field);
     f.register('checkbox', field.checkbox_field);
     f.register('checkboxes', field.field);
     f.register('combobox', field.field);
@@ -1529,6 +1671,8 @@ field.register = function() {
     v.register('same_password', field.same_password_validator);
 
     l.register('adapter', field.Adapter);
+    l.register('object_adapter', field.ObjectAdapter);
+    l.register('alternate_attr_field_adapter', field.AlternateAttrFieldAdapter);
 };
 phases.on('registration', field.register);
 
