@@ -104,14 +104,9 @@ return {
                 'title'
             ],
             adder_columns:[
-                {
-                    name: 'cn',
-                    width: '100px'
-                },
                 {
                     name: 'uid',
-                    primary_key: true,
-                    width: '100px'
+                    primary_key: true
                 }
             ]
         },
