@@ -28,13 +28,14 @@ define([
         './facets/Facet',
         './topology_graph',
         './navigation',
+        './widget',
         // plain imports
         './search',
         './entity'],
             function(lang, declare, Evented, Stateful, Deferred, on, all, when,
                 builder, IPA, $, menu, metadata_provider, phases, reg, rpc,
                 text, mod_details, mod_facet, mod_field, ActionMixin,
-                HeaderMixin, Facet, topology_graph, navigation) {
+                HeaderMixin, Facet, topology_graph, navigation, widget_mod) {
 /**
  * Topology module
  * @class
@@ -50,8 +51,10 @@ var topology = IPA.topology = {
         facets: {
             suffix_search: 'topologysuffix_search',
             server_search: 'server_search',
+            server_role_search: 'server_role_search',
             domainlevel: 'domainlevel_details',
-            topologygraph: 'topology-graph'
+            topologygraph: 'topology-graph',
+            location_search: 'location_search'
         }
     }
 };
@@ -193,9 +196,18 @@ return {
 var make_server_spec = function() {
 return {
     name: 'server',
+    policies: [
+        {
+            $factory: IPA.facet_update_policy,
+            source_facet: 'details',
+            dest_entity: 'location',
+            dest_facet: 'details'
+        }
+    ],
     facets: [
            {
             $type: 'search',
+            $factory: topology.servers_search_facet,
             no_update: true,
             disable_facet_tabs: false,
             tabs_in_sidebar: true,
@@ -212,7 +224,7 @@ return {
         },
         {
             $type: 'details',
-            no_update: true,
+            $factory: topology.servers_facet,
             disable_facet_tabs: true,
             sections: [
                 {
@@ -221,9 +233,102 @@ return {
                         { name: 'cn', read_only: true },
                         { name: 'ipamindomainlevel', read_only: true },
                         { name: 'ipamaxdomainlevel', read_only: true },
-                        { $type: 'multivalued', name: 'iparepltopomanagedsuffix_topologysuffix', read_only: true }
+                        { $type: 'multivalued', name: 'iparepltopomanagedsuffix_topologysuffix', read_only: true },
+                        {
+                            $type: 'entity_select',
+                            name: 'ipalocation_location',
+                            other_entity: 'location',
+                            other_field: 'idnsname',
+                            flags: ['w_if_no_aci']
+                        },
+                        {
+                            name: 'ipaserviceweight',
+                            placeholder: '100',
+                            flags: ['w_if_no_aci']
+                        },
+                        {
+                            $type: 'association_table',
+                            other_entity: 'server_role',
+                            name: 'enabled_role_servrole',
+                            footer: false,
+                            read_only: true,
+                            selectable: false
+                        }
                     ]
                 }
+            ],
+            actions: [
+                {
+                    $factory: IPA.delete_action,
+                    name: 'server_del',
+                    method: 'del',
+                    label: '@i18n:objects.servers.remove_server',
+                    needs_confirm: true,
+                    confirm_msg: '@i18n:objects.servers.remove_server_msg',
+                    confirm_dialog: {
+                        $factory: IPA.confirm_dialog,
+                        title: '@i18n:objects.servers.remove_server',
+                        ok_label: '@i18n:buttons.remove',
+                        ok_button_class: 'btn btn-danger'
+                    }
+                }
+            ],
+            control_buttons_right: [
+                {
+                    name: 'server_del',
+                    label: '@i18n:objects.servers.remove_server',
+                    icon: 'fa-exclamation-circle',
+                    button_class: 'btn btn-danger'
+                }
+            ]
+        }
+    ]
+};};
+
+
+var make_serverroles_spec = function() {
+return {
+    name: 'server_role',
+    facet_groups: [ 'role_servers' ],
+    facets: [
+        {
+            $factory: topology.serverroles_search_facet,
+            $type: 'search',
+            primary_key_name: 'role_servrole',
+            search_all_entries: true,
+            disable_search_field: true,
+            no_update: true,
+            selectable: false,
+            disable_facet_tabs: false,
+            tabs_in_sidebar: true,
+            tab_label: '@i18n:objects.server_role.label',
+            label: '@i18n:objects.server_role.label',
+            facet_groups: [topology.search_facet_group],
+            facet_group: 'search',
+            columns: [
+                'role_servrole',
+                'status'
+            ]
+        },
+        {
+            $factory: topology.serverroles_nested_search_facet,
+            $type: 'nested_search',
+            primary_key_name: 'server_server',
+            search_all_entries: true,
+            disable_search_field: true,
+            no_update: true,
+            selectable: false,
+            nested_entity: 'server_role',
+            facet_group: 'role_servers',
+            label: '@i18n:objects.server_role.label_singular',
+            columns: [
+                {
+                    name: 'server_server',
+                    target_entity: 'server',
+                    target_facet: 'details',
+                    link: true
+                },
+                'status'
             ]
         }
     ]
@@ -272,6 +377,398 @@ topology.domainlevel_adapter = declare([mod_field.Adapter], {
 });
 
 
+var make_location_spec = function() {
+return {
+    name: 'location',
+    policies: [
+        {
+            $factory: IPA.facet_update_policy,
+            source_facet: 'details',
+            dest_entity: 'server',
+            dest_facet: 'details'
+        },
+        {
+            $factory: IPA.facet_update_policy,
+            source_facet: 'details',
+            dest_entity: 'location',
+            dest_facet: 'search'
+        }
+    ],
+    facets: [
+        {
+            $type: 'search',
+            disable_facet_tabs: false,
+            tabs_in_sidebar: true,
+            tab_label: '@mo:location.label',
+            facet_groups: [topology.search_facet_group],
+            facet_group: 'search',
+            columns: [
+                'idnsname',
+                'description'
+            ]
+        },
+        {
+            $type: 'details',
+            disable_facet_tabs: true,
+            section_layout_class: 'col-sm-12',
+            sections: [
+                {
+                    name: 'general',
+                    label: '@i18n:details.general',
+                    fields: [
+                        'idnsname',
+                        {
+                            $type: 'textarea',
+                            name: 'description'
+                        }
+                    ]
+                },
+                {
+                    $factory: IPA.section,
+                    name: 'servers',
+                    label: '@mo:server.label',
+                    fields: [
+                        {
+                            $type: 'location_association_table',
+                            adapter: topology.location_adapter,
+                            other_entity: 'server',
+                            footer: false,
+                            name: 'cn',
+                            columns: [
+                                {
+                                    name: 'cn',
+                                    link: true
+                                },
+                                {
+                                    name: 'ipaserviceweight'
+                                },
+                                {
+                                    name: 'service_relative_weight'
+                                }
+                            ]
+                        }
+                    ]
+                }
+            ]
+        }
+    ],
+    adder_dialog: {
+        fields: [
+            {
+                $type: 'text',
+                name: 'idnsname',
+                required: true
+            },
+            'description'
+        ]
+    }
+};};
+
+topology.location_adapter = declare([mod_field.Adapter], {
+    load: function(data) {
+        var result = this.get_record(data);
+        if (result.servers_server === undefined) return [];
+
+        result = result.servers_server;
+
+        this.result_name = 'servers';
+        var servers = this.get_record(data);
+        this.result_name = 'result';
+
+        var output = [];
+
+        for (var i=0, l=result.length; i<l; i++) {
+            output.push(servers[result[i]]);
+        }
+
+        return output;
+    }
+});
+
+topology.servers_search_facet = function(spec, no_init) {
+    spec = spec || {};
+
+    var that = IPA.search_facet(spec);
+
+    that.create_get_records_command = function(pkeys, on_success, on_error) {
+
+        var on_success_extended = function(data, text_status, xhr) {
+            // Call original on_success handler
+            on_success(data, text_status, xhr);
+
+            var result = data.result.results;
+            var counter = 0;
+
+            for (var i=0, l=result.length; i<l; i++) {
+                var current = result[i];
+                var roles = current.result.enabled_role_servrole;
+                for (var k=0, m=roles.length; k<m; k++) {
+                    if (roles[k] === 'CA server') counter++;
+                }
+            }
+
+            // Create dialog and show it only when there is only one CA server
+            if (counter != 1) return;
+
+            var message = text.get('@i18n:objects.servers.ca_warning_message');
+            var dialog = IPA.dialog({
+                name: 'ca_warning',
+                title: '@i18n:objects.servers.ca_warning_title',
+                sections: [
+                    {
+                        show_header: false,
+                        layout:
+                        {
+                            $factory: widget_mod.fluid_layout,
+                            widget_cls: "col-sm-12 controls",
+                            label_cls: "hide"
+                        },
+                        fields: [
+                            {
+                                field: false,
+                                $type: 'html',
+                                html: message
+                            }
+                        ]
+                    }
+                ]
+            });
+
+            dialog.create_button({
+                name: 'ok',
+                label: '@i18n:buttons.ok',
+                click: function() {
+                    dialog.close();
+                }
+            });
+
+            dialog.open();
+        };
+
+        var batch = that.table_facet_create_get_records_command(pkeys,
+                                                on_success_extended, on_error);
+
+        return batch;
+    };
+
+    return that;
+};
+
+topology.servers_facet = function(spec, no_init) {
+    spec = spec || {};
+
+    var that = IPA.details_facet(spec, no_init);
+
+    /**
+     * Creates buttons on the right side of facet.
+     */
+    that.create_controls = function() {
+
+        that.create_control_buttons(that.controls_left);
+        that.create_action_dropdown(that.controls_left);
+
+        that.control_buttons = that.control_buttons_right;
+        that.create_control_buttons(that.controls_right);
+    };
+
+    /**
+     * Inits right facet buttons.
+     */
+    that.init_servers_facet = function() {
+
+        that.init_details_facet();
+        var buttons_spec = {
+            $factory: IPA.control_buttons_widget,
+            name: 'control-buttons',
+            css_class: 'control-buttons',
+            buttons: spec.control_buttons_right
+        };
+
+        that.control_buttons_right = IPA.build(buttons_spec);
+        that.control_buttons_right.init(that);
+    };
+
+    if (!no_init) that.init_servers_facet();
+
+    return that;
+};
+
+topology.serverroles_search_facet = function(spec) {
+
+    spec = spec || {};
+
+    var that = IPA.search_facet(spec);
+
+    that.filter_records = function(records_map, pkey, record) {
+
+        var stored_record = records_map.get(pkey);
+        if (!stored_record) return true;
+
+        // set priority to all possible values. Value with higher priority
+        // will overwrite a value with lower prio.
+        var priority_map = {
+            absent: 0,
+            configured: 1,
+            enabled: 2
+        };
+
+        var priority_old = priority_map[stored_record.status];
+        var priority_new = priority_map[record.status];
+        return priority_new >= priority_old;
+    };
+
+    that.get_refresh_command_args = function() {
+        // We need to call find command with no argument.
+        return;
+    };
+
+    return that;
+};
+
+topology.serverroles_nested_search_facet = function(spec) {
+
+    spec = spec || {};
+    spec.disable_facet_tabs = true;
+
+    var that = IPA.nested_search_facet(spec);
+
+    that.get_refresh_command_args = function() {
+        // We need to call find command with no argument.
+        return;
+    };
+
+    that.get_refresh_command_options = function() {
+        return { 'role_servrole': that.get_pkey() };
+    };
+
+    that.on_column_link_click = function(value, entity) {
+        var pkeys = [value];
+
+        navigation.show_entity('server', that.details_facet_name, pkeys);
+        return false;
+    };
+
+    that.filter_records = function(records_map, pkey, record) {
+        // Return false when the status is 'absent'
+        return record.status !== 'absent';
+    };
+
+    return that;
+};
+
+topology.location_server_adder_dialog = function(spec) {
+    spec = spec || {};
+
+    var that = IPA.entity_adder_dialog(spec);
+
+    that.init = function() {
+        that.added.attach(function() {
+            that.facet.refresh();
+            that.facet.on_update.notify();
+        });
+    };
+
+    that.create_add_command = function(record) {
+        var pkey = that.facet.get_pkey();
+        var command = that.entity_adder_dialog_create_add_command(record);
+
+        command.set_option('ipalocation_location', pkey);
+
+        return command;
+    };
+
+    that.init();
+
+    return that;
+};
+
+topology.location_association_table_widget = function(spec) {
+
+    spec = spec || {};
+    spec.name = spec.name || 'servers';
+
+    var that = IPA.association_table_widget(spec);
+
+    that.create_add_dialog = function() {
+
+        var entity_label = that.entity.metadata.label_singular;
+        var pkey = that.facet.get_pkey();
+        var other_entity_label = that.other_entity.metadata.label_singular;
+
+        var title = that.add_title;
+        title = title.replace('${entity}', entity_label);
+        title = title.replace('${primary_key}', pkey);
+        title = title.replace('${other_entity}', other_entity_label);
+
+        return topology.location_server_adder_dialog({
+            title: title,
+            entity: that.other_entity.name,
+            method: 'mod',
+            options: {
+                'ipalocation_location': pkey
+            },
+            sections: [
+                {
+                    fields: [
+                        {
+                            $type: 'entity_select',
+                            name: 'cn',
+                            required: true,
+                            other_entity: that.other_entity.name,
+                            other_field: 'cn',
+                            filter_options: {
+                                'not_in_location': pkey
+                            }
+                        },
+                        {
+                            $type: 'text',
+                            name: 'ipaserviceweight'
+                        }
+                    ]
+                }
+            ]
+        });
+    };
+
+
+    that.remove = function(values, on_success, on_error) {
+
+        var pkey = '';
+
+        var batch = rpc.batch_command({
+            on_success: function(data) {
+                that.remove_button.set_enabled(false);
+                that.facet.refresh();
+
+                var count = data.result.count;
+                var msg = text.get('@i18n:association.removed').replace('${count}', count);
+                IPA.notify_success(msg);
+            },
+            on_error: on_error
+        });
+
+        var command, value;
+        for(var i = 0, l=values.length; i<l; i++) {
+            value = values[i];
+
+            command = rpc.command({
+                entity: that.other_entity.name,
+                method: 'mod',
+                args: [value],
+                options: {
+                    'ipalocation_location': pkey
+                }
+            });
+
+            batch.add_command(command);
+        }
+
+        batch.execute();
+    };
+
+    return that;
+};
+
 topology.domainlevel_metadata = function(spec, context) {
     var metadata = metadata_provider.source;
     metadata.objects.domainlevel = {
@@ -433,16 +930,22 @@ topology.TopologyGraphFacet = declare([Facet, ActionMixin, HeaderMixin], {
         var graph = this.get_widget('topology-graph');
         var listener = this.resize_listener.bind(this, graph);
 
-        on(this, 'show', lang.hitch(this, function(args) {
+        on(this, 'show', function(args) {
             var size = this.calculate_canvas_size();
             graph.update(size);
 
             $(window).on('resize', null, listener);
-        }));
+        }.bind(this));
 
-        on(graph, 'link-selected', lang.hitch(this, function(data) {
+        on(graph, 'link-selected', function(data) {
             this.set_selected_link(data.link);
-        }));
+        }.bind(this));
+
+        on(graph, 'add-agreement', function(link) {
+            var dialog = topology.create_add_dialog();
+            dialog.open();
+            dialog.prefill_dialog(link);
+        });
 
         on(this, 'hide', function () {
             $(window).off('resize', null, listener);
@@ -478,6 +981,107 @@ topology.TopologyGraphFacet = declare([Facet, ActionMixin, HeaderMixin], {
     }
 });
 
+
+/**
+ * Creates segment adder dialog according to spec.
+ *
+ * @param spec {Object}
+ */
+topology.create_add_dialog = function(spec) {
+
+    spec = spec || {};
+
+    spec.entity = spec.entity || 'topologysegment';
+
+    var entity = reg.entity.get('topologysegment');
+    var title = text.get('@i18n:dialogs.add_title');
+    var label = entity.metadata.label_singular;
+    spec.title = title.replace('${entity}', label);
+
+    spec.fields = spec.fields || [
+        {
+            name: 'cn',
+            placeholder: '@i18n:objects.topology.autogenerated',
+            required: false
+        },
+        {
+            $type: 'entity_select',
+            name: 'suffix',
+            label: '@mo:topologysuffix.label_singular',
+            other_entity: 'topologysuffix',
+            other_field: 'cn',
+            z_index: 3,
+            required: true
+        },
+        {
+            $type: 'entity_select',
+            name: 'iparepltoposegmentleftnode',
+            other_entity: 'server',
+            other_field: 'cn',
+            z_index: 2
+        },
+        {
+            $type: 'entity_select',
+            name: 'iparepltoposegmentrightnode',
+            other_entity: 'server',
+            other_field: 'cn',
+            z_index: 1
+        }
+    ];
+
+
+    var that = IPA.entity_adder_dialog(spec);
+
+    that.init = function() {
+        that.added.attach(that.on_success);
+    };
+
+    that.show_edit_page = function(entity, result) {
+        var suffix = this.fields.get_field('suffix').save()[0];
+        var cn = result.cn[0];
+        navigation.show_entity(entity.name, 'default', [suffix, cn]);
+    };
+
+    that.create_add_command = function(record) {
+
+        var args = [this.fields.get_field('suffix').save()[0]];
+        var cn = this.fields.get_field('cn').save()[0];
+        if (cn) args.push(cn);
+
+        var options = {
+            'iparepltoposegmentleftnode':
+                this.fields.get_field('iparepltoposegmentleftnode').save()[0],
+            'iparepltoposegmentrightnode':
+                this.fields.get_field('iparepltoposegmentrightnode').save()[0]
+        };
+
+        var command = rpc.command({
+            entity: this.entity.name,
+            method: this.method,
+            retry: this.retry,
+            args: args,
+            options: options
+        });
+
+        return command;
+    };
+
+
+    that.on_success = function(data) {
+        that.facet.refresh();
+    };
+
+    that.prefill_dialog = function(link) {
+        that.get_field('iparepltoposegmentleftnode').set_value([link.source.id]);
+        that.get_field('iparepltoposegmentrightnode').set_value([link.target.id]);
+        that.get_field('suffix').set_value([link.suffix]);
+    };
+
+    that.init();
+
+    return that;
+};
+
 /**
  * Shows topology segment adder dialog with suffix select
  *
@@ -495,84 +1099,11 @@ topology.add_segment_action = function(spec) {
 
     that.execute_action = function(facet, on_success, on_error) {
 
-        that.facet = facet;
-
-        var entity = reg.entity.get('topologysegment');
-        var title = text.get('@i18n:dialogs.add_title');
-        var label = entity.metadata.label_singular;
-        title = title.replace('${entity}', label);
-
-        var dialog = IPA.entity_adder_dialog({
-            entity: 'topologysegment',
-            title: title,
-            fields: [
-                {
-                    name: 'cn',
-                    required: false
-                },
-                {
-                    $type: 'entity_select',
-                    name: 'suffix',
-                    label: '@mo:topologysuffix.label_singular',
-                    other_entity: 'topologysuffix',
-                    other_field: 'cn',
-                    z_index: 3,
-                    required: true
-                },
-                {
-                    $type: 'entity_select',
-                    name: 'iparepltoposegmentleftnode',
-                    other_entity: 'server',
-                    other_field: 'cn',
-                    z_index: 2
-                },
-                {
-                    $type: 'entity_select',
-                    name: 'iparepltoposegmentrightnode',
-                    other_entity: 'server',
-                    other_field: 'cn',
-                    z_index: 1
-                }
-            ]
-        });
-        dialog.added.attach(that.on_success);
-
-        dialog.show_edit_page = function(entity, result) {
-            var suffix = this.fields.get_field('suffix').save()[0];
-            var cn = result.cn[0];
-            navigation.show_entity(entity.name, 'default', [suffix, cn]);
-        };
-
-        dialog.create_add_command = function(record) {
-
-            var args = [this.fields.get_field('suffix').save()[0]];
-            var cn = this.fields.get_field('cn').save()[0];
-            if (cn) args.push(cn);
-
-            var options = {
-                'iparepltoposegmentleftnode':
-                    this.fields.get_field('iparepltoposegmentleftnode').save()[0],
-                'iparepltoposegmentrightnode':
-                    this.fields.get_field('iparepltoposegmentrightnode').save()[0]
-            };
-
-            var command = rpc.command({
-                entity: this.entity.name,
-                method: this.method,
-                retry: this.retry,
-                args: args,
-                options: options
-            });
-
-            return command;
-        };
+        var dialog = topology.create_add_dialog();
 
         dialog.open();
     };
 
-    that.on_success = function(data) {
-        that.facet.refresh();
-    };
 
     return that;
 };
@@ -662,11 +1193,11 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
                 sizelimit: 0
             }
         }).execute();
-        when(s_promise, lang.hitch(this, function(results) {
+        when(s_promise, function(results) {
             // suffixes load success
             var servers = results.data.result.result;
             deferred.resolve(servers);
-        }), function(results) {
+        }.bind(this), function(results) {
             deferred.reject({
                 message: 'unable to load servers',
                 results: results
@@ -701,7 +1232,7 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
 
         var suff_promise = get_suffixes();
 
-        when(suff_promise, lang.hitch(this, function(results) {
+        when(suff_promise, function(results) {
             // suffixes load success
             var suffixes = results.data.result.result;
             var segment_promises = [];
@@ -710,26 +1241,26 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
                 var promise = get_segments(suffix['cn'][0]);
                 segment_promises.push(promise);
             }
-            all(segment_promises).then(lang.hitch(this, function(results) {
+            all(segment_promises).then(function(results) {
                 // segments load success
                 for (var j=0,l=results.length; j<l; j++) {
                     suffixes[j].segments = results[j].data.result.result;
                 }
                 deferred.resolve(suffixes);
-            }), lang.hitch(this, function(results) {
+            }.bind(this), function(results) {
                 // segments load failed
                 deferred.reject({
                     message: 'unable to load segments',
                     results: results
                 });
-            }));
-        }), lang.hitch(this, function(results) {
+            }.bind(this));
+        }.bind(this), function(results) {
             // suffix load failed
             deferred.reject({
                 message: 'unable to load suffixes',
                 results: results
             });
-        }));
+        }.bind(this));
 
         return deferred.promise;
     },
@@ -818,10 +1349,10 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
         var segments = this._get_suffixes();
         var masters = this._get_servers();
 
-        all([masters, segments]).then(lang.hitch(this, function(raw) {
+        all([masters, segments]).then(function(raw) {
             var data = this._transform_data(raw[0], raw[1]);
             deferred.resolve(data);
-        }), function(error) {
+        }.bind(this), function(error) {
             deferred.reject(error);
         });
 
@@ -836,7 +1367,7 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
 
         if (IPA.domain_level < topology.required_domain_level) return;
 
-        when(this._get_data()).then(lang.hitch(this, function(data) {
+        when(this._get_data()).then(function(data) {
             if (!this.graph) {
                 this.graph = new topology_graph.TopoGraph({
                     nodes: data.nodes,
@@ -854,7 +1385,7 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
                 }
                 this.graph.update(data.nodes, data.links, data.suffixes);
             }
-        }), function(error) {
+        }.bind(this), function(error) {
             IPA.notify(error.message, 'error');
         });
     },
@@ -869,6 +1400,7 @@ topology.TopologyGraphWidget = declare([Stateful, Evented], {
         }
 
         forward('link-selected');
+        forward('add-agreement');
     },
 
     _update_view: function() {
@@ -964,12 +1496,20 @@ topology.segment_spec = make_segment_spec();
  */
 topology.server_spec = make_server_spec();
 
+/**
+ * IPA server roles entity specification object
+ * @member topology
+ */
+topology.serverroles_spec = make_serverroles_spec();
+
+
 /**
  * Domain Level entity specification object
  * @member topology
  */
 topology.domainlevel_spec = make_domainlevel_spec();
 
+topology.location_spec = make_location_spec();
 
 /**
  * Register entity
@@ -984,13 +1524,16 @@ topology.register = function() {
     e.register({type: 'topologysuffix', spec: topology.suffix_spec});
     e.register({type: 'topologysegment', spec: topology.segment_spec});
     e.register({type: 'server', spec: topology.server_spec});
+    e.register({type: 'server_role', spec: topology.serverroles_spec});
     e.register({type: 'domainlevel', spec: topology.domainlevel_spec});
+    e.register({type: 'location', spec: topology.location_spec});
 
     a.register('domainlevel_set', topology.domainlevel_set_action);
     a.register('segment_add', topology.add_segment_action);
     a.register('segment_del', topology.del_segment_action);
 
     w.register('topology-graph', topology.TopologyGraphWidget);
+    w.register('location_association_table', topology.location_association_table_widget);
     fa.register({
         type: 'topology-graph',
         ctor: topology.TopologyGraphFacet,
