@@ -36,7 +36,8 @@ Operating System :: Unix
 
 # BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
 # update it when the contents of directories change.
-if os.path.exists('MANIFEST'): os.remove('MANIFEST')
+if os.path.exists('MANIFEST'):
+    os.remove('MANIFEST')
 
 def setup_package():
 
@@ -50,7 +51,7 @@ def setup_package():
     try:
         setup(
             name = "ipaclient",
-            version = "4.3.2",
+            version = "4.4.3",
             license = "GPL",
             url = "http://www.freeipa.org/",
             description = DOCLINES[0],
@@ -59,8 +60,15 @@ def setup_package():
             classifiers=[line for line in CLASSIFIERS.split('\n') if line],
             platforms = ["Linux", "Solaris", "Unix"],
             package_dir = {'ipaclient': ''},
-            packages = ["ipaclient",
-                       ],
+            packages = [
+                "ipaclient",
+                "ipaclient.plugins",
+                "ipaclient.remote_plugins",
+                "ipaclient.remote_plugins.2_49",
+                "ipaclient.remote_plugins.2_114",
+                "ipaclient.remote_plugins.2_156",
+                "ipaclient.remote_plugins.2_164",
+            ],
             scripts=['../ipa'],
             data_files = [('share/man/man1', ["../ipa.1"])],
         )
