@@ -22,6 +22,7 @@ Functionality for Command Line Interface.
 """
 from __future__ import print_function
 
+import importlib
 import textwrap
 import sys
 import getpass
@@ -52,8 +53,7 @@ from ipalib.errors import (PublicError, CommandError, HelpError, InternalError,
 from ipalib.constants import CLI_TAB, LDAP_GENERALIZED_TIME_FORMAT
 from ipalib.parameters import File, Str, Enum, Any, Flag
 from ipalib.text import _
-from ipalib import api
-from ipapython.version import API_VERSION
+from ipalib import api  # pylint: disable=unused-import
 from ipapython.dnsutil import DNSName
 
 import datetime
@@ -324,6 +324,7 @@ class textui(backend.Backend):
                     if len(text) == 0:
                         text = [u'']
                 else:
+                    s_indent = u''
                     text = [text]
                 self.print_indented(format % (attr, text[0]), indent)
                 for line in text[1:]:
@@ -365,8 +366,6 @@ class textui(backend.Backend):
             self.print_entry(entry, order, labels, flags, print_all, format, indent)
 
     def print_entry(self, entry, order=None, labels=None, flags=None, print_all=True, format='%s: %s', indent=1):
-        """
-        """
         if isinstance(entry, (list, tuple)):
             entry = dict(entry)
         assert isinstance(entry, dict)
@@ -667,7 +666,7 @@ class textui(backend.Backend):
                 selection = int(resp) - 1
                 if (selection >= 0 and selection < counter):
                     break
-            except:
+            except Exception:
                 # fall through to the error msg
                 pass
 
@@ -691,28 +690,30 @@ class help(frontend.Local):
 
     has_output = tuple()
 
-    _PLUGIN_BASE_MODULE = 'ipalib.plugins'
+    topic = None
 
-    def _get_command_module(self, module):
-        """
-        Return last part of ``module`` name, or ``None`` if module is this file.
+    def _get_topic(self, topic):
+        doc = u''
+        parent_topic = None
 
-        For example:
-        """
-        if module == __name__:
-            return
-        return module.split('.')[-1]
+        for package in self.api.packages:
+            module_name = '%s.%s' % (package.__name__, topic)
+            try:
+                module = sys.modules[module_name]
+            except KeyError:
+                try:
+                    module = importlib.import_module(module_name)
+                except ImportError:
+                    continue
 
-    def _get_module_topic(self, module_name):
-        if not sys.modules[module_name]:
-            __import__(module_name)
-        module = sys.modules[module_name]
+            if module.__doc__ is not None:
+                doc = unicode(module.__doc__ or '').strip()
+            try:
+                parent_topic = module.topic
+            except AttributeError:
+                pass
 
-        topic = getattr(module, 'topic', None)
-        if topic is None:
-            topic = (self._get_command_module(module_name), None)
-
-        return topic
+        return doc, parent_topic
 
     def _count_topic_mcl(self, topic_name, mod_name):
         mcl = max((self._topics[topic_name][1], len(mod_name)))
@@ -726,35 +727,26 @@ class help(frontend.Local):
         self._builtins = []
 
         # build help topics
-        for c in self.api.Command():
+        for c in self.api.Command:
+            if c is not self.api.Command.get_plugin(c.name):
+                continue
             if c.NO_CLI:
                 continue
 
-            topic = self._get_module_topic(c.module)
-            topic_name = topic[0]
-
-            if topic_name:
-                if topic[1] is None: # a module without grouping
+            if c.topic is not None:
+                doc, topic_name = self._get_topic(c.topic)
+                doc = doc.split('\n', 1)[0]
+                if topic_name is None:  # a module without grouping
+                    topic_name = c.topic
                     if topic_name in self._topics:
                         self._topics[topic_name][2].append(c)
                     else:
-                        m = '%s.%s' % (self._PLUGIN_BASE_MODULE, topic_name)
-                        try:
-                            module = sys.modules[m]
-                        except KeyError:
-                            doc = ''
-                        else:
-                            doc = (
-                                unicode(_(module.__doc__)) or ''
-                            ).strip().split('\n', 1)[0]
                         self._topics[topic_name] = [doc, 0, [c]]
                     mcl = max((self._topics[topic_name][1], len(c.name)))
                     self._topics[topic_name][1] = mcl
                 else: # a module grouped in a topic
-                    doc = (
-                        unicode(_(sys.modules[c.module].__doc__)) or ''
-                    ).strip().split('\n', 1)[0]
-                    mod_name = c.module.rsplit('.',1)[1]
+                    topic = self._get_topic(topic_name)
+                    mod_name = c.topic
                     if topic_name in self._topics:
                         if mod_name in self._topics[topic_name][2]:
                             self._topics[topic_name][2][mod_name][2].append(c)
@@ -765,7 +757,9 @@ class help(frontend.Local):
                         mcl = max((self._topics[topic_name][2][mod_name][1], len(c.name)))
                         self._topics[topic_name][2][mod_name][1] = mcl
                     else:
-                        self._topics[topic_name] = [unicode(_(topic[1])), 0, {mod_name: [doc, 0, [c]]}]
+                        self._topics[topic_name] = [topic[0].split('\n', 1)[0],
+                                                    0,
+                                                    {mod_name: [doc, 0, [c]]}]
                         self._count_topic_mcl(topic_name, mod_name)
             else:
                 self._builtins.append(c)
@@ -776,12 +770,11 @@ class help(frontend.Local):
 
         super(help, self)._on_finalize()
 
-    def run(self, key, outfile=None, **options):
+    def run(self, key=None, outfile=None, **options):
         if outfile is None:
             outfile = sys.stdout
         writer = self._writer(outfile)
         name = from_cli(key)
-        mod_name = '%s.%s' % (self._PLUGIN_BASE_MODULE, name)
         if key is None:
             self.api.parser.print_help(outfile)
             return
@@ -795,15 +788,19 @@ class help(frontend.Local):
             if cmd.NO_CLI:
                 raise HelpError(topic=name)
             self.Backend.cli.build_parser(cmd).print_help(outfile)
-        elif mod_name in sys.modules:
+        elif any(name in t[2] for t in self._topics.values()
+                 if type(t[2]) is dict):
             self.print_commands(name, outfile)
         elif name == "commands":
-            mcl = max(len(s) for s in (self.Command))
-            for cname in self.Command:
-                cmd = self.Command[cname]
-                if cmd.NO_CLI:
+            mcl = 0
+            for cmd_plugin in self.Command:
+                if cmd_plugin is not self.Command.get_plugin(cmd_plugin.name):
                     continue
-                writer('%s  %s' % (to_cli(cmd.name).ljust(mcl), cmd.summary))
+                if cmd_plugin.NO_CLI:
+                    continue
+                mcl = max(mcl, len(cmd_plugin.name))
+                writer('%s  %s' % (to_cli(cmd_plugin.name).ljust(mcl),
+                                   cmd_plugin.summary))
         else:
             raise HelpError(topic=name)
 
@@ -845,8 +842,7 @@ class help(frontend.Local):
                     commands = self._topics[t][2][topic][2]
                     break
 
-            m = '%s.%s' % (self._PLUGIN_BASE_MODULE, topic)
-            doc = (unicode(_(sys.modules[m].__doc__)) or '').strip()
+            doc, _topic = self._get_topic(topic)
 
             if topic not in self.Command and len(commands) == 0:
                 raise HelpError(topic=topic)
@@ -874,6 +870,8 @@ class show_mappings(frontend.Command):
     )
     has_output = tuple()
 
+    topic = None
+
     def run(self, command_name, **options):
         command_name = from_cli(command_name)
         if command_name not in self.Command:
@@ -901,6 +899,8 @@ class console(frontend.Command):
     takes_args = ('filename?',)
     has_output = tuple()
 
+    topic = None
+
     def run(self, filename=None, **options):
         local = dict(api=self.api)
         if filename:
@@ -911,7 +911,7 @@ class console(frontend.Command):
             try:
                 with script:
                     exec(script, globals(), local)
-            except:
+            except Exception:
                 traceback.print_exc()
                 exit(1)
         else:
@@ -926,7 +926,9 @@ class show_api(frontend.Command):
 
     takes_args = ('namespaces*',)
 
-    def run(self, namespaces):
+    topic = None
+
+    def run(self, namespaces=None):
         if namespaces is None:
             names = tuple(self.api)
         else:
@@ -970,7 +972,7 @@ class show_api(frontend.Command):
                 continue
             for n in member:
                 attr = member[n]
-                if isinstance(attr, plugable.NameSpace) and len(attr) > 0:
+                if isinstance(attr, plugable.APINameSpace) and len(attr) > 0:
                     self.__traverse_namespace(n, attr, lines, tab + 2)
 
 
@@ -1099,7 +1101,7 @@ class cli(backend.Executioner):
         cmd = self.get_command(argv)
         if cmd is None:
             return
-        name = cmd.name
+        name = cmd.full_name
         kw = self.parse(cmd, argv[1:])
         if not isinstance(cmd, frontend.Local):
             self.create_context()
@@ -1167,10 +1169,8 @@ class cli(backend.Executioner):
                     kw['action'] = 'store_false'
                 else:
                     kw['action'] = 'store_true'
-            elif isinstance(option, Enum):
-                kw['metavar'] = list(str(x) for x in option.values)
             else:
-                kw['metavar'] = option.__class__.__name__.upper()
+                kw['metavar'] = option.cli_metavar
 
             cli_name = to_cli(option.cli_name)
             option_names = ['--%s' % cli_name]
@@ -1246,6 +1246,8 @@ class cli(backend.Executioner):
                 if param.autofill:
                     kw[param.name] = cmd.get_default_of(param.name, **kw)
                 if param.name in kw and kw[param.name] is not None:
+                    if param.autofill:
+                        del kw[param.name]
                     continue
                 if param.password:
                     kw[param.name] = self.Backend.textui.prompt_password(
