@@ -28,10 +28,10 @@ from ipapython.version import VERSION, API_VERSION
 
 try:
     FQDN = socket.getfqdn()
-except:
+except Exception:
     try:
         FQDN = socket.gethostname()
-    except:
+    except Exception:
         FQDN = None
 
 # Path to CA certificate bundle
@@ -121,6 +121,10 @@ DEFAULT_CONFIG = (
     ('container_certprofile', DN(('cn', 'certprofiles'), ('cn', 'ca'))),
     ('container_topology', DN(('cn', 'topology'), ('cn', 'ipa'), ('cn', 'etc'))),
     ('container_caacl', DN(('cn', 'caacls'), ('cn', 'ca'))),
+    ('container_locations', DN(('cn', 'locations'), ('cn', 'etc'))),
+    ('container_ca', DN(('cn', 'cas'), ('cn', 'ca'))),
+    ('container_dnsservers', DN(('cn', 'servers'), ('cn', 'dns'))),
+    ('container_custodia', DN(('cn', 'custodia'), ('cn', 'ipa'), ('cn', 'etc'))),
 
     # Ports, hosts, and URIs:
     ('xmlrpc_uri', 'http://localhost:8888/ipa/xml'),
@@ -129,6 +133,8 @@ DEFAULT_CONFIG = (
 
     ('rpc_protocol', 'jsonrpc'),
 
+    ('nss_dir', paths.IPA_NSSDB_DIR),
+
     # Define an inclusive range of SSL/TLS version support
     ('tls_version_min', 'tls1.0'),
     ('tls_version_max', 'tls1.2'),
@@ -189,6 +195,9 @@ DEFAULT_CONFIG = (
     # behavior when newer clients talk to older servers. Use with caution.
     ('skip_version_check', False),
 
+    # Ignore TTL. Perform schema call and download schema if not in cache.
+    ('force_schema_check', False),
+
     # ********************************************************
     #  The remaining keys are never set from the values here!
     # ********************************************************
@@ -226,6 +235,7 @@ DEFAULT_CONFIG = (
     ('logdir', object),  # Directory containing log files
     ('log', object),  # Path to context specific log file
     ('jsonrpc_uri', object),  # derived from xmlrpc_uri in Env._finalize_core()
+    ('server', object),  # derived from jsonrpc_uri in Env._finalize_core()
 
 )
 
@@ -261,4 +271,11 @@ REPL_AGMT_STRIP_ATTRS = ('modifiersName',
 
 DOMAIN_SUFFIX_NAME = 'domain'
 CA_SUFFIX_NAME = 'ca'
+PKI_GSSAPI_SERVICE_NAME = 'dogtag'
+IPA_CA_CN = u'ipa'
 IPA_CA_RECORD = "ipa-ca"
+IPA_CA_NICKNAME = 'caSigningCert cert-pki-ca'
+RENEWAL_CA_NAME = 'dogtag-ipa-ca-renew-agent'
+
+# regexp definitions
+PATTERN_GROUPUSER_NAME = '^[a-zA-Z0-9_.][a-zA-Z0-9_.-]*[a-zA-Z0-9_.$-]?$'
