@@ -1,7 +1,7 @@
 # Authors:
 #   Jason Gerard DeRose <jderose@redhat.com>
 #
-# Copyright (C) 2008  Red Hat
+# Copyright (C) 2008-2016  Red Hat
 # see file 'COPYING' for use and warranty inmsgion
 #
 # This program is free software; you can redistribute it and/or modify
@@ -108,7 +108,7 @@ import six
 
 from ipalib.text import ngettext as ungettext
 from ipalib import messages
-from ipaplatform.paths import paths
+from ipaplatform.paths import paths  # pylint: disable=unused-import
 
 
 class PrivateError(Exception):
@@ -238,6 +238,14 @@ class PluginsPackageError(PrivateError):
     format = '%(name)s must be sub-package, not module: %(file)r'
 
 
+class PluginModuleError(PrivateError):
+    """
+    Raised when a module is not a valid plugin module.
+    """
+
+    format = '%(name)s is not a valid plugin module'
+
+
 ##############################################################################
 # Public errors:
 
@@ -399,7 +407,7 @@ class ServerNetworkError(PublicError):
 
 class JSONError(PublicError):
     """
-    **909** Raised when server recieved a malformed JSON-RPC request.
+    **909** Raised when server received a malformed JSON-RPC request.
     """
 
     errno = 909
@@ -467,7 +475,7 @@ class KerberosError(AuthenticationError):
 
 class CCacheError(KerberosError):
     """
-    **1101** Raised when sever does not recieve Kerberose credentials.
+    **1101** Raised when sever does not receive Kerberose credentials.
 
     For example:
 
@@ -603,6 +611,18 @@ class PasswordExpired(InvalidSessionPassword):
     """
     errno = 1202
 
+class KrbPrincipalExpired(SessionError):
+    """
+    **1203** Raised when Kerberos Principal is expired.
+    """
+    errno = 1203
+
+class UserLocked(SessionError):
+    """
+    **1204** Raised when a user account is locked.
+    """
+    errno = 1204
+
 ##############################################################################
 # 2000 - 2999: Authorization errors
 class AuthorizationError(PublicError):
@@ -846,7 +866,6 @@ class NotAForestRootError(InvocationError):
     errno = 3016
     format = _("Domain '%(domain)s' is not a root domain for forest '%(forest)s'")
 
-
 ##############################################################################
 # 4000 - 4999: Execution errors
 
@@ -1359,6 +1378,44 @@ class InvalidDomainLevelError(ExecutionError):
     errno = 4032
     format = _('%(reason)s')
 
+
+class ServerRemovalError(ExecutionError):
+    """
+    **4033** Raised when a removal of IPA server from managed topology fails
+
+    For example:
+
+    >>> raise ServerRemovalError(reason='Removal disconnects topology')
+    Traceback (most recent call last):
+      ...
+    ServerRemovalError: Server removal aborted: Removal disconnects topology
+
+    """
+
+    errno = 4033
+    format = _('Server removal aborted: %(reason)s.')
+
+
+class OperationNotSupportedForPrincipalType(ExecutionError):
+    """
+    **4034** Raised when an operation is not supported for a principal type
+    """
+
+    errno = 4034
+    format = _(
+        '%(operation)s is not supported for %(principal_type)s principals')
+
+
+class HTTPRequestError(RemoteRetrieveError):
+    """
+    **4035** Raised when an HTTP request fails.  Includes the response
+    status in the ``status`` attribute.
+    """
+
+    errno = 4035
+    format = _('Request failed with status %(status)s: %(reason)s')
+
+
 class BuiltinError(ExecutionError):
     """
     **4100** Base class for builtin execution errors (*4100 - 4199*).
@@ -1581,6 +1638,34 @@ class TaskTimeout(DatabaseError):
     format = _("%(task)s LDAP task timeout, Task DN: '%(task_dn)s'")
 
 
+class TimeLimitExceeded(LimitsExceeded):
+    """
+    **4214** Raised when time limit for the operation is exceeded.
+    """
+
+    errno = 4214
+    format = _('Configured time limit exceeded')
+
+
+class SizeLimitExceeded(LimitsExceeded):
+    """
+    **4215** Raised when size limit for the operation is exceeded.
+    """
+
+    errno = 4215
+    format = _('Configured size limit exceeded')
+
+
+class AdminLimitExceeded(LimitsExceeded):
+    """
+    **4216** Raised when server limit imposed by administrative authority was
+    exceeded
+    """
+
+    errno = 4216
+    format = _('Configured administrative server limit exceeded')
+
+
 class CertificateError(ExecutionError):
     """
     **4300** Base class for Certificate execution errors (*4300 - 4399*).
@@ -1754,6 +1839,24 @@ class CertificateInvalidError(CertificateError):
     format = _('%(name)s certificate is not valid')
 
 
+class SchemaUpToDate(ExecutionError):
+    """
+    **4311** Raised by server when client asks for metadata but
+    already has current version. Exception's attribute 'fingerprint'
+    identitfies schema version to use. Attribute 'ttl' specifies how
+    long (in seconds) before client should check for schema update.
+
+    For example:
+    >>> raise SchemaUpToDate(fingerprint=u'deadbeef', ttl=3600)
+    Traceback (most recent call last):
+      ...
+    SchemaUpToDate: Schema is up to date (FP 'deadbeef', TTL 3600 s)
+    """
+
+    errno = 4311
+    format = _("Schema is up to date (FP '%(fingerprint)s', TTL %(ttl)s s)")
+
+
 class DNSError(ExecutionError):
     """
     **4400** Base class for DNS execution errors (*4400 - 4499*).
@@ -1814,6 +1917,34 @@ class DNSResolverError(DNSError):
     errno = 4401
     format = _('%(exception)s')
 
+class TrustError(ExecutionError):
+    """
+    **4500** Base class for trust execution errors (*4500 - 4599*).
+    These are typically instantiated when there is an error in establishing or
+    modifying a trust to another forest.
+    """
+
+    errno = 4500
+
+class TrustTopologyConflictError(TrustError):
+    """
+    **4501** Raised when an attempt to establish trust fails with a topology
+             conflict against another forest the target forest trusts
+
+    For example:
+
+    >>> raise TrustTopologyConflictError(forest='example.test',
+                                         conflict='my.ad.test',
+                                         domains=['ad.test'])
+    Traceback (most recent call last):
+      ...
+    TrustTopologyConflictError: Forest 'example.test' has existing trust to forest(s) ['ad.test'] which prevents a trust to 'my.ad.test'
+    """
+
+    errno = 4501
+    format = _("Forest '%(forest)s' has existing trust to forest(s) "
+               "%(domains)s which prevents a trust to '%(conflict)s'")
+
 
 ##############################################################################
 # 5000 - 5999: Generic errors
