@@ -25,7 +25,6 @@ DOCLINES = __doc__.split("\n")
 
 import os
 import sys
-import distutils.sysconfig
 
 CLASSIFIERS = """\
 Intended Audience :: System Environment/Base
@@ -37,7 +36,8 @@ Operating System :: Unix
 
 # BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
 # update it when the contents of directories change.
-if os.path.exists('MANIFEST'): os.remove('MANIFEST')
+if os.path.exists('MANIFEST'):
+    os.remove('MANIFEST')
 
 def setup_package():
 
@@ -51,7 +51,7 @@ def setup_package():
     try:
         setup(
             name = "ipalib",
-            version = "4.3.2",
+            version = "4.4.3",
             license = "GPL",
             url = "http://www.freeipa.org/",
             description = DOCLINES[0],
@@ -60,9 +60,7 @@ def setup_package():
             classifiers=[line for line in CLASSIFIERS.split('\n') if line],
             platforms = ["Linux", "Solaris", "Unix"],
             package_dir = {'ipalib': ''},
-            packages = ["ipalib",
-                        "ipalib.plugins",
-                       ],
+            packages = ["ipalib"],
         )
     finally:
         del sys.path[0]
