@@ -21,14 +21,18 @@
 Various utility functions.
 """
 
+from __future__ import (
+    absolute_import,
+    print_function,
+)
+
 import os
-import imp
-import time
 import socket
 import re
 import decimal
 import dns
 import encodings
+import sys
 from weakref import WeakKeyDictionary
 
 import netaddr
@@ -45,7 +49,7 @@ from ipapython.ssh import SSHPublicKey
 from ipapython.dn import DN, RDN
 from ipapython.dnsutil import DNSName
 from ipapython.dnsutil import resolve_ip_addresses
-from ipapython.graph import Graph
+from ipapython.ipa_log_manager import root_logger
 
 if six.PY3:
     unicode = str
@@ -195,9 +199,16 @@ def validate_dns_label(dns_label, allow_underscore=False, allow_slash=False):
 
     middle_chars = middle_chars + '-' #has to be always the last in the regex [....-]
 
-    label_regex = r'^[%(base)s%(extra)s]([%(base)s%(extra)s%(middle)s]?[%(base)s%(extra)s])*$' \
-        % dict(base=base_chars, extra=extra_chars, middle=middle_chars)
-    regex = re.compile(label_regex, re.IGNORECASE)
+    label_regex = r'''^[%(base)s%(extra)s] # must begin with an alphanumeric
+                                           # character, or underscore if
+                                           # allow_underscore is True
+        ([%(base)s%(extra)s%(middle)s]*    # can contain all allowed character
+                                           # classes in the middle
+        [%(base)s%(extra)s])*$             # must end with alphanumeric
+                                           # character or underscore if
+                                           # allow_underscore is True
+        ''' % dict(base=base_chars, extra=extra_chars, middle=middle_chars)
+    regex = re.compile(label_regex, re.IGNORECASE | re.VERBOSE)
 
     if not dns_label:
         raise ValueError(_('empty DNS label'))
@@ -276,12 +287,9 @@ def validate_sshpubkey_no_options(ugettext, value):
     if pubkey.has_options():
         return _('options are not allowed')
 
-def convert_sshpubkey_post(ldap, dn, entry_attrs):
-    if 'ipasshpubkey' in entry_attrs:
-        pubkeys = entry_attrs['ipasshpubkey']
-    else:
-        old_entry_attrs = ldap.get_entry(dn, ['ipasshpubkey'])
-        pubkeys = old_entry_attrs.get('ipasshpubkey')
+
+def convert_sshpubkey_post(entry_attrs):
+    pubkeys = entry_attrs.get('ipasshpubkey')
     if not pubkeys:
         return
 
@@ -307,6 +315,37 @@ def convert_sshpubkey_post(ldap, dn, entry_attrs):
     if fingerprints:
         entry_attrs['sshpubkeyfp'] = fingerprints
 
+
+def add_sshpubkey_to_attrs_pre(context, attrs_list):
+    """
+    Attribute ipasshpubkey should be added to attrs_list to be able compute
+    ssh fingerprint. This attribute must be removed later if was added here
+    (see remove_sshpubkey_from_output_post).
+    """
+    if not ('ipasshpubkey' in attrs_list or '*' in attrs_list):
+        setattr(context, 'ipasshpubkey_added', True)
+        attrs_list.append('ipasshpubkey')
+
+
+def remove_sshpubkey_from_output_post(context, entry_attrs):
+    """
+    Remove ipasshpubkey from output if it was added in pre_callbacks
+    """
+    if getattr(context, 'ipasshpubkey_added', False):
+        entry_attrs.pop('ipasshpubkey', None)
+        delattr(context, 'ipasshpubkey_added')
+
+
+def remove_sshpubkey_from_output_list_post(context, entries):
+    """
+    Remove ipasshpubkey from output if it was added in pre_callbacks
+    """
+    if getattr(context, 'ipasshpubkey_added', False):
+        for entry_attrs in entries:
+            entry_attrs.pop('ipasshpubkey', None)
+        delattr(context, 'ipasshpubkey_added')
+
+
 class cachedproperty(object):
     """
     A property-like attribute that caches the return value of a method call.
@@ -737,55 +776,6 @@ def validate_idna_domain(value):
         raise ValueError(error)
 
 
-def create_topology_graph(masters, segments):
-    """
-    Create an oriented graph from topology defined by masters and segments.
-
-    :param masters
-    :param segments
-    :returns: Graph
-    """
-    graph = Graph()
-
-    for m in masters:
-        graph.add_vertex(m['cn'][0])
-
-    for s in segments:
-        direction = s['iparepltoposegmentdirection'][0]
-        left = s['iparepltoposegmentleftnode'][0]
-        right = s['iparepltoposegmentrightnode'][0]
-        try:
-            if direction == u'both':
-                graph.add_edge(left, right)
-                graph.add_edge(right, left)
-            elif direction == u'left-right':
-                graph.add_edge(left, right)
-            elif direction == u'right-left':
-                graph.add_edge(right, left)
-        except ValueError:  # ignore segments with deleted master
-            pass
-
-    return graph
-
-
-def get_topology_connection_errors(graph):
-    """
-    Traverse graph from each master and find out which masters are not
-    reachable.
-
-    :param graph: topology graph where vertices are masters
-    :returns: list of errors, error is: (master, visited, not_visited)
-    """
-    connect_errors = []
-    master_cns = list(graph.vertices)
-    master_cns.sort()
-    for m in master_cns:
-        visited = graph.bfs(m)
-        not_visited = graph.vertices - visited
-        if not_visited:
-            connect_errors.append((m, list(visited), list(not_visited)))
-    return connect_errors
-
 def detect_dns_zone_realm_type(api, domain):
     """
     Detects the type of the realm that the given DNS zone belongs to.
@@ -846,9 +836,186 @@ def has_managed_topology(api):
     return domainlevel > DOMAIN_LEVEL_0
 
 
+class classproperty(object):
+    __slots__ = ('__doc__', 'fget')
+
+    def __init__(self, fget=None, doc=None):
+        if doc is None and fget is not None:
+            doc = fget.__doc__
+
+        self.fget = fget
+        self.__doc__ = doc
+
+    def __get__(self, obj, obj_type):
+        if self.fget is not None:
+            return self.fget.__get__(obj, obj_type)()
+        raise AttributeError("unreadable attribute")
+
+    def __set__(self, obj, value):
+        raise AttributeError("can't set attribute")
+
+    def __delete__(self, obj):
+        raise AttributeError("can't delete attribute")
+
+    def getter(self, fget):
+        self.fget = fget
+        return self
+
+
 def normalize_hostname(hostname):
     """Use common fqdn form without the trailing dot"""
     if hostname.endswith(u'.'):
         hostname = hostname[:-1]
     hostname = hostname.lower()
     return hostname
+
+
+def hostname_validator(ugettext, value):
+    try:
+        validate_hostname(value)
+    except ValueError as e:
+        return _('invalid domain-name: %s') % unicode(e)
+
+    return None
+
+
+def ipaddr_validator(ugettext, ipaddr, ip_version=None):
+    try:
+        ip = netaddr.IPAddress(str(ipaddr), flags=netaddr.INET_PTON)
+
+        if ip_version is not None:
+            if ip.version != ip_version:
+                return _(
+                    'invalid IP address version (is %(value)d, must be '
+                    '%(required_value)d)!') % dict(
+                    value=ip.version,
+                    required_value=ip_version
+                )
+    except (netaddr.AddrFormatError, ValueError):
+        return _('invalid IP address format')
+    return None
+
+
+def validate_bind_forwarder(ugettext, forwarder):
+    ip_address, sep, port = forwarder.partition(u' port ')
+
+    ip_address_validation = ipaddr_validator(ugettext, ip_address)
+
+    if ip_address_validation is not None:
+        return ip_address_validation
+
+    if sep:
+        try:
+            port = int(port)
+            if port < 0 or port > 65535:
+                raise ValueError()
+        except ValueError:
+            return _('%(port)s is not a valid port' % dict(port=port))
+
+    return None
+
+
+def set_krbcanonicalname(entry_attrs):
+    objectclasses = set(i.lower() for i in entry_attrs['objectclass'])
+
+    if 'krbprincipalaux' not in objectclasses:
+        return
+
+    if ('krbprincipalname' in entry_attrs
+            and 'krbcanonicalname' not in entry_attrs):
+        entry_attrs['krbcanonicalname'] = entry_attrs['krbprincipalname']
+
+
+def ensure_last_krbprincipalname(ldap, entry_attrs, *keys):
+    """
+    ensure that the LDAP entry has at least one value of krbprincipalname
+    and that this value is equal to krbcanonicalname
+
+    :param ldap: LDAP connection object
+    :param entry_attrs: LDAP entry made prior to update
+    :param options: command options
+    """
+    entry = ldap.get_entry(
+        entry_attrs.dn, ['krbcanonicalname', 'krbprincipalname'])
+
+    krbcanonicalname = entry.single_value.get('krbcanonicalname', None)
+
+    if krbcanonicalname in keys[-1]:
+        raise errors.ValidationError(
+            name='krbprincipalname',
+            error=_('at least one value equal to the canonical '
+                    'principal name must be present')
+        )
+
+
+def ensure_krbcanonicalname_set(ldap, entry_attrs):
+    old_entry = ldap.get_entry(
+        entry_attrs.dn,
+        ['krbcanonicalname', 'krbprincipalname', 'objectclass'])
+
+    if old_entry.single_value.get('krbcanonicalname', None) is not None:
+        return
+
+    set_krbcanonicalname(old_entry)
+
+    old_entry.pop('krbprincipalname', None)
+    old_entry.pop('objectclass', None)
+
+    entry_attrs.update(old_entry)
+
+
+def check_principal_realm_in_trust_namespace(api_instance, *keys):
+    """
+    Check that principal name's suffix does not overlap with UPNs and realm
+    names of trusted forests.
+
+    :param api_instance: API instance
+    :param suffixes: principal suffixes
+
+    :raises: ValidationError if the suffix coincides with realm name, UPN
+    suffix or netbios name of trusted domains
+    """
+    trust_objects = api_instance.Command.trust_find(u'', sizelimit=0)['result']
+
+    trust_suffix_namespace = set()
+
+    for obj in trust_objects:
+        nt_suffixes = obj.get('ipantadditionalsuffixes', [])
+
+        trust_suffix_namespace.update(
+            set(upn.lower() for upn in nt_suffixes))
+
+        if 'ipantflatname' in obj:
+            trust_suffix_namespace.add(obj['ipantflatname'][0].lower())
+
+        trust_suffix_namespace.add(obj['cn'][0].lower())
+
+    for principal in keys[-1]:
+        realm = principal.realm
+        upn = principal.upn_suffix if principal.is_enterprise else None
+
+        if realm in trust_suffix_namespace or upn in trust_suffix_namespace:
+            raise errors.ValidationError(
+                name='krbprincipalname',
+                error=_('realm or UPN suffix overlaps with trusted domain '
+                        'namespace'))
+
+
+def network_ip_address_warning(addr_list):
+    for ip in addr_list:
+        if ip.is_network_addr():
+            root_logger.warning("IP address %s might be network address", ip)
+            # fixme: once when loggers will be fixed, we can remove this
+            # print
+            print("WARNING: IP address {} might be network address".format(ip),
+                  file=sys.stderr)
+
+
+def broadcast_ip_address_warning(addr_list):
+    for ip in addr_list:
+        if ip.is_broadcast_addr():
+            root_logger.warning("IP address %s might be broadcast address", ip)
+            # fixme: once when loggers will be fixed, we can remove this
+            # print
+            print("WARNING: IP address {} might be broadcast address".format(
+                ip), file=sys.stderr)
