@@ -19,10 +19,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import string
 import time
 import datetime
-import shutil
 from decimal import Decimal
 from copy import deepcopy
 import contextlib
@@ -33,19 +31,18 @@ import pwd
 import ldap
 import ldap.sasl
 import ldap.filter
-from ldap.ldapobject import SimpleLDAPObject
 from ldap.controls import SimplePagedResultsControl
 import ldapurl
 import six
 
 from ipalib import errors, _
 from ipalib.constants import LDAP_GENERALIZED_TIME_FORMAT
-from ipapython import ipautil
 from ipapython.ipautil import (
     format_netloc, wait_for_open_socket, wait_for_open_ports, CIDict)
 from ipapython.ipa_log_manager import log_mgr
-from ipapython.dn import DN, RDN
+from ipapython.dn import DN
 from ipapython.dnsutil import DNSName
+from ipapython.kerberos import Principal
 
 if six.PY3:
     unicode = str
@@ -63,6 +60,11 @@ AUTOBIND_AUTO = 1
 AUTOBIND_ENABLED = 2
 AUTOBIND_DISABLED = 3
 
+TRUNCATED_SIZE_LIMIT = object()
+TRUNCATED_TIME_LIMIT = object()
+TRUNCATED_ADMIN_LIMIT = object()
+
+
 def unicode_from_utf8(val):
     '''
     val is a UTF-8 encoded string, return a unicode object.
@@ -153,7 +155,6 @@ class SchemaCache(object):
         If a connection is provided then it the credentials bound to it are
         used. The connection is not closed when the request is done.
         """
-        tmpdir = None
         assert conn is not None
 
         self.log.debug(
@@ -182,9 +183,6 @@ class SchemaCache(object):
             # TODO: DS uses 'cn=schema', support for other server?
             #       raise a more appropriate exception
             raise
-        finally:
-            if tmpdir:
-                shutil.rmtree(tmpdir)
 
         return ldap.schema.SubSchema(schema_entry[1])
 
@@ -311,7 +309,11 @@ class LDAPEntry(collections.MutableMapping):
             raw.remove(value)
 
         for value in raw_dels:
-            value = self._conn.decode(value, name)
+            try:
+                value = self._conn.decode(value, name)
+            except ValueError as e:
+                raise ValueError("{error} in LDAP entry '{dn}'".format(
+                    error=e, dn=self._dn))
             if value in nice_adds:
                 continue
             nice.remove(value)
@@ -323,7 +325,11 @@ class LDAPEntry(collections.MutableMapping):
             raw.append(value)
 
         for value in raw_adds:
-            value = self._conn.decode(value, name)
+            try:
+                value = self._conn.decode(value, name)
+            except ValueError as e:
+                raise ValueError("{error} in LDAP entry '{dn}'".format(
+                    error=e, dn=self._dn))
             if value in nice_dels:
                 continue
             nice.append(value)
@@ -681,6 +687,8 @@ class LDAPClient(object):
         'idnssoamname':    DNSName,
         'idnssoarname':    DNSName,
         'dnszoneidnsname': DNSName,
+        'krbcanonicalname': Principal,
+        'krbprincipalname': Principal,
         'nsds5replicalastupdatestart': unicode,
         'nsds5replicalastupdateend': unicode,
         'nsds5replicalastinitstart': unicode,
@@ -842,7 +850,8 @@ class LDAPClient(object):
                 return 'TRUE'
             else:
                 return 'FALSE'
-        elif isinstance(val, (unicode, six.integer_types, Decimal, DN)):
+        elif isinstance(val, (unicode, six.integer_types, Decimal, DN,
+                              Principal)):
             return value_to_utf8(val)
         elif isinstance(val, DNSName):
             return val.to_text()
@@ -974,11 +983,11 @@ class LDAPClient(object):
         except ldap.OBJECT_CLASS_VIOLATION:
             raise errors.ObjectclassViolation(info=info)
         except ldap.ADMINLIMIT_EXCEEDED:
-            raise errors.LimitsExceeded()
+            raise errors.AdminLimitExceeded()
         except ldap.SIZELIMIT_EXCEEDED:
-            raise errors.LimitsExceeded()
+            raise errors.SizeLimitExceeded()
         except ldap.TIMELIMIT_EXCEEDED:
-            raise errors.LimitsExceeded()
+            raise errors.TimeLimitExceeded()
         except ldap.NOT_ALLOWED_ON_RDN:
             raise errors.NotAllowedOnRDN(attr=info)
         except ldap.FILTER_ERROR:
@@ -1006,6 +1015,20 @@ class LDAPClient(object):
                 'Unhandled LDAPError: %s: %s' % (type(e).__name__, str(e)))
             raise errors.DatabaseError(desc=desc, info=info)
 
+    @staticmethod
+    def handle_truncated_result(truncated):
+        if not truncated:
+            return
+
+        if truncated is TRUNCATED_ADMIN_LIMIT:
+            raise errors.AdminLimitExceeded()
+        elif truncated is TRUNCATED_SIZE_LIMIT:
+            raise errors.SizeLimitExceeded()
+        elif truncated is TRUNCATED_TIME_LIMIT:
+            raise errors.TimeLimitExceeded()
+        else:
+            raise errors.LimitsExceeded()
+
     @property
     def schema(self):
         """schema associated with this LDAP server"""
@@ -1134,7 +1157,8 @@ class LDAPClient(object):
     # entry_attrs = {u'firstName': u'Pavel', u'lastName': u'Zuna'}
     # f = ldap2.make_filter(entry_attrs, rules=ldap2.MATCH_ALL)
 
-    def combine_filters(self, filters, rules='|'):
+    @classmethod
+    def combine_filters(cls, filters, rules='|'):
         """
         Combine filters into one for ldap2.find_entries.
 
@@ -1145,9 +1169,9 @@ class LDAPClient(object):
         assert isinstance(filters, (list, tuple))
 
         filters = [f for f in filters if f]
-        if filters and rules == self.MATCH_NONE:  # unary operator
-            return '(%s%s)' % (self.MATCH_NONE,
-                               self.combine_filters(filters, self.MATCH_ANY))
+        if filters and rules == cls.MATCH_NONE:  # unary operator
+            return '(%s%s)' % (cls.MATCH_NONE,
+                               cls.combine_filters(filters, cls.MATCH_ANY))
 
         if len(filters) > 1:
             flt = '(%s' % rules
@@ -1161,8 +1185,9 @@ class LDAPClient(object):
             flt = '%s)' % flt
         return flt
 
+    @classmethod
     def make_filter_from_attr(
-            self, attr, value, rules='|', exact=True,
+            cls, attr, value, rules='|', exact=True,
             leading_wildcard=True, trailing_wildcard=True):
         """
         Make filter for ldap2.find_entries from attribute.
@@ -1179,20 +1204,25 @@ class LDAPClient(object):
             False - forbid trailing filter wildcard when exact=False
         """
         if isinstance(value, (list, tuple)):
-            if rules == self.MATCH_NONE:
-                make_filter_rules = self.MATCH_ANY
+            if rules == cls.MATCH_NONE:
+                make_filter_rules = cls.MATCH_ANY
             else:
                 make_filter_rules = rules
             flts = [
-                self.make_filter_from_attr(
+                cls.make_filter_from_attr(
                     attr, v, exact=exact,
                     leading_wildcard=leading_wildcard,
                     trailing_wildcard=trailing_wildcard)
                 for v in value
             ]
-            return self.combine_filters(flts, rules)
+            return cls.combine_filters(flts, rules)
         elif value is not None:
-            value = ldap.filter.escape_filter_chars(value_to_utf8(value))
+            if isinstance(value, bytes):
+                if six.PY3:
+                    value = value.decode('raw_unicode_escape')
+            else:
+                value = value_to_utf8(value)
+            value = ldap.filter.escape_filter_chars(value)
             if not exact:
                 template = '%s'
                 if leading_wildcard:
@@ -1200,13 +1230,14 @@ class LDAPClient(object):
                 if trailing_wildcard:
                     template = template + '*'
                 value = template % value
-            if rules == self.MATCH_NONE:
+            if rules == cls.MATCH_NONE:
                 return '(!(%s=%s))' % (attr, value)
             return '(%s=%s)' % (attr, value)
         return ''
 
+    @classmethod
     def make_filter(
-            self, entry_attrs, attrs_list=None, rules='|', exact=True,
+            cls, entry_attrs, attrs_list=None, rules='|', exact=True,
             leading_wildcard=True, trailing_wildcard=True):
         """
         Make filter for ldap2.find_entries from entry attributes.
@@ -1228,15 +1259,15 @@ class LDAPClient(object):
         ldap2.MATCH_ALL - match entries that match all attributes
         ldap2.MATCH_ANY - match entries that match any of attribute
         """
-        if rules == self.MATCH_NONE:
-            make_filter_rules = self.MATCH_ANY
+        if rules == cls.MATCH_NONE:
+            make_filter_rules = cls.MATCH_ANY
         else:
             make_filter_rules = rules
         flts = []
         if attrs_list is None:
             for (k, v) in entry_attrs.items():
                 flts.append(
-                    self.make_filter_from_attr(
+                    cls.make_filter_from_attr(
                         k, v, make_filter_rules, exact,
                         leading_wildcard, trailing_wildcard)
                 )
@@ -1245,14 +1276,14 @@ class LDAPClient(object):
                 value = entry_attrs.get(a, None)
                 if value is not None:
                     flts.append(
-                        self.make_filter_from_attr(
+                        cls.make_filter_from_attr(
                             a, value, make_filter_rules, exact,
                             leading_wildcard, trailing_wildcard)
                     )
-        return self.combine_filters(flts, rules)
+        return cls.combine_filters(flts, rules)
 
     def get_entries(self, base_dn, scope=ldap.SCOPE_SUBTREE, filter=None,
-                    attrs_list=None):
+                    attrs_list=None, **kwargs):
         """Return a list of matching entries.
 
         :raises: errors.LimitsExceeded if the list is truncated by the server
@@ -1263,13 +1294,21 @@ class LDAPClient(object):
         :param scope: search scope, see LDAP docs (default ldap2.SCOPE_SUBTREE)
         :param filter: LDAP filter to apply
         :param attrs_list: ist of attributes to return, all if None (default)
-
-        Use the find_entries method for more options.
+        :param kwargs: additional keyword arguments. See find_entries method
+        for their description.
         """
         entries, truncated = self.find_entries(
             base_dn=base_dn, scope=scope, filter=filter, attrs_list=attrs_list)
-        if truncated:
-            raise errors.LimitsExceeded()
+        try:
+            self.handle_truncated_result(truncated)
+        except errors.LimitsExceeded as e:
+            self.log.error(
+                "{} while getting entries (base DN: {}, filter: {})".format(
+                    e, base_dn, filter
+                )
+            )
+            raise
+
         return entries
 
     def find_entries(self, filter=None, attrs_list=None, base_dn=None,
@@ -1360,6 +1399,15 @@ class LDAPClient(object):
                                 break
                         else:
                             cookie = ''
+                except ldap.ADMINLIMIT_EXCEEDED:
+                    truncated = TRUNCATED_ADMIN_LIMIT
+                    break
+                except ldap.SIZELIMIT_EXCEEDED:
+                    truncated = TRUNCATED_SIZE_LIMIT
+                    break
+                except ldap.TIMELIMIT_EXCEEDED:
+                    truncated = TRUNCATED_TIME_LIMIT
+                    break
                 except ldap.LDAPError as e:
                     # If paged search is in progress, try to cancel it
                     if paged_search and cookie:
@@ -1405,15 +1453,13 @@ class LDAPClient(object):
 
         search_kw = {attr: value, 'objectClass': object_class}
         filter = self.make_filter(search_kw, rules=self.MATCH_ALL)
-        (entries, truncated) = self.find_entries(filter, attrs_list, base_dn)
+        entries = self.get_entries(
+            base_dn, filter=filter, attrs_list=attrs_list)
 
         if len(entries) > 1:
             raise errors.SingleMatchExpected(found=len(entries))
-        else:
-            if truncated:
-                raise errors.LimitsExceeded()
-            else:
-                return entries[0]
+
+        return entries[0]
 
     def get_entry(self, dn, attrs_list=None, time_limit=None,
                   size_limit=None):
@@ -1426,13 +1472,11 @@ class LDAPClient(object):
 
         assert isinstance(dn, DN)
 
-        (entries, truncated) = self.find_entries(
-            None, attrs_list, dn, self.SCOPE_BASE, time_limit=time_limit,
+        entries = self.get_entries(
+            dn, self.SCOPE_BASE, None, attrs_list, time_limit=time_limit,
             size_limit=size_limit
         )
 
-        if truncated:
-            raise errors.LimitsExceeded()
         return entries[0]
 
     def add_entry(self, entry):
