@@ -1,6 +1,6 @@
 # Authors: Simo Sorce <ssorce@redhat.com>
 #
-# Copyright (C) 2007-2011  Red Hat
+# Copyright (C) 2007-2016  Red Hat, Inc.
 # see file 'COPYING' for use and warranty information
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,15 +23,16 @@ import string
 import tempfile
 import subprocess
 import random
-import os, sys, traceback
+import os
+import sys
 import copy
 import stat
 import shutil
 import socket
-import struct
 import re
 import datetime
 import netaddr
+import netifaces
 import time
 import gssapi
 import pwd
@@ -39,15 +40,16 @@ import grp
 from contextlib import contextmanager
 import locale
 import collections
+from subprocess import CalledProcessError
 
-from dns import resolver, rdatatype, reversename
+from dns import resolver, reversename
 from dns.exception import DNSException
+
 import six
 from six.moves import input
 from six.moves import urllib
 
-from ipapython.ipa_log_manager import *
-from ipapython import ipavalidate
+from ipapython.ipa_log_manager import root_logger
 from ipapython import config
 from ipaplatform.paths import paths
 from ipapython.dn import DN
@@ -55,27 +57,14 @@ from ipapython.dn import DN
 SHARE_DIR = paths.USR_SHARE_IPA_DIR
 PLUGINS_SHARE_DIR = paths.IPA_PLUGINS
 
-GEN_PWD_LEN = 12
+GEN_PWD_LEN = 22
+GEN_TMP_PWD_LEN = 12  # only for OTP password that is manually retyped by user
 
 # Having this in krb_utils would cause circular import
 KRB5_KDC_UNREACH = 2529639068 # Cannot contact any KDC for requested realm
 KRB5KDC_ERR_SVC_UNAVAILABLE = 2529638941 # A service is not available that is
                                          # required to process the request
 
-try:
-    from subprocess import CalledProcessError
-except ImportError:
-    # Python 2.4 doesn't implement CalledProcessError
-    class CalledProcessError(Exception):
-        """This exception is raised when a process run by check_call() returns
-        a non-zero exit status. The exit status will be stored in the
-        returncode attribute."""
-        def __init__(self, returncode, cmd, output=None):
-            self.returncode = returncode
-            self.cmd = cmd
-            self.output = output
-        def __str__(self):
-            return "Command '%s' returned non-zero exit status %d" % (self.cmd, self.returncode)
 
 def get_domain_name():
     try:
@@ -136,6 +125,17 @@ class UnsafeIPAddress(netaddr.IPAddress):
         super(UnsafeIPAddress, self).__init__(addr,
                                               flags=self.netaddr_ip_flags)
 
+    def __getstate__(self):
+        state = {
+            '_net': self._net,
+            'super_state': super(UnsafeIPAddress, self).__getstate__(),
+        }
+        return state
+
+    def __setstate__(self, state):
+        super(UnsafeIPAddress, self).__setstate__(state['super_state'])
+        self._net = state['_net']
+
 
 class CheckedIPAddress(UnsafeIPAddress):
     """IPv4 or IPv6 address with additional constraints.
@@ -143,8 +143,7 @@ class CheckedIPAddress(UnsafeIPAddress):
     Reserved or link-local addresses are never accepted.
     """
     def __init__(self, addr, match_local=False, parse_netmask=True,
-                 allow_network=False, allow_loopback=False,
-                 allow_broadcast=False, allow_multicast=False):
+                 allow_loopback=False, allow_multicast=False):
 
         super(CheckedIPAddress, self).__init__(addr)
         if isinstance(addr, CheckedIPAddress):
@@ -173,25 +172,31 @@ class CheckedIPAddress(UnsafeIPAddress):
 
         if match_local:
             if self.version == 4:
-                family = 'inet'
+                family = netifaces.AF_INET
             elif self.version == 6:
-                family = 'inet6'
+                family = netifaces.AF_INET6
+            else:
+                raise ValueError(
+                    "Unsupported address family ({})".format(self.version)
+                )
 
-            result = run(
-                [paths.IP, '-family', family, '-oneline', 'address', 'show'],
-                capture_output=True)
-            lines = result.output.split('\n')
             iface = None
-            for line in lines:
-                fields = line.split()
-                if len(fields) < 4:
-                    continue
+            for interface in netifaces.interfaces():
+                for ifdata in netifaces.ifaddresses(interface).get(family, []):
 
-                ifnet = netaddr.IPNetwork(fields[3])
-                if ifnet == self._net or (self._net is None and ifnet.ip == self):
-                    self._net = ifnet
-                    iface = fields[1]
-                    break
+                    # link-local addresses contain '%suffix' that causes parse
+                    # errors in IPNetwork
+                    ifaddr = ifdata['addr'].split(u'%', 1)[0]
+
+                    ifnet = netaddr.IPNetwork('{addr}/{netmask}'.format(
+                        addr=ifaddr,
+                        netmask=ifdata['netmask']
+                    ))
+                    if ifnet == self._net or (
+                            self._net is None and ifnet.ip == self):
+                        self._net = ifnet
+                        iface = interface
+                        break
 
             if iface is None:
                 raise ValueError('no network interface matches the IP address '
@@ -204,14 +209,25 @@ class CheckedIPAddress(UnsafeIPAddress):
             elif self.version == 6:
                 self._net = netaddr.IPNetwork(str(self) + '/64')
 
-        if not allow_network and self == self._net.network:
-            raise ValueError("cannot use IP network address {}".format(addr))
-        if not allow_broadcast and (self.version == 4 and
-                                    self == self._net.broadcast):
-            raise ValueError("cannot use broadcast IP address {}".format(addr))
-
         self.prefixlen = self._net.prefixlen
 
+    def __getstate__(self):
+        state = {
+            'prefixlen': self.prefixlen,
+            'super_state': super(CheckedIPAddress, self).__getstate__(),
+        }
+        return state
+
+    def __setstate__(self, state):
+        super(CheckedIPAddress, self).__setstate__(state['super_state'])
+        self.prefixlen = state['prefixlen']
+
+    def is_network_addr(self):
+        return self == self._net.network
+
+    def is_broadcast_addr(self):
+        return self.version == 4 and self == self._net.broadcast
+
 
 def valid_ip(addr):
     return netaddr.valid_ipv4(addr) or netaddr.valid_ipv6(addr)
@@ -526,7 +542,7 @@ def file_exists(filename):
             return True
         else:
             return False
-    except:
+    except Exception:
         return False
 
 def dir_exists(filename):
@@ -536,7 +552,7 @@ def dir_exists(filename):
             return True
         else:
             return False
-    except:
+    except Exception:
         return False
 
 
@@ -1339,7 +1355,8 @@ def kinit_keytab(principal, keytab, ccache_name, config=None, attempts=1):
 
 
 def kinit_password(principal, password, ccache_name, config=None,
-                   armor_ccache_name=None):
+                   armor_ccache_name=None, canonicalize=False,
+                   enterprise=False):
     """
     perform interactive kinit as principal using password. If using FAST for
     web-based authentication, use armor_ccache_path to specify http service
@@ -1352,6 +1369,14 @@ def kinit_password(principal, password, ccache_name, config=None,
                           % armor_ccache_name)
         args.extend(['-T', armor_ccache_name])
 
+    if canonicalize:
+        root_logger.debug("Requesting principal canonicalization")
+        args.append('-C')
+
+    if enterprise:
+        root_logger.debug("Using enterprise principal")
+        args.append('-E')
+
     env = {'LC_ALL': 'C'}
     if config is not None:
         env['KRB5_CONFIG'] = config
@@ -1394,18 +1419,6 @@ def dn_attribute_property(private_name):
 
     return property(getter, setter)
 
-def restore_hostname(statestore):
-    """
-    Restore hostname of a machine, if it was set before
-    """
-    old_hostname = statestore.restore_state('network','hostname')
-    system_hostname = socket.gethostname()
-    if old_hostname is not None and old_hostname != system_hostname:
-        try:
-            run([paths.BIN_HOSTNAME, old_hostname])
-        except CalledProcessError as e:
-            print("Failed to set this machine hostname back to %s: %s" % (old_hostname, str(e)), file=sys.stderr)
-
 def posixify(string):
     """
     Convert a string to a more strict alpha-numeric representation.
@@ -1496,3 +1509,30 @@ def is_fips_enabled():
         # Consider that the host is not fips-enabled if the file does not exist
         pass
     return False
+
+
+def unescape_seq(seq, *args):
+    """
+    unescape (remove '\\') all occurences of sequence in input strings.
+
+    :param seq: sequence to unescape
+    :param args: input string to process
+
+    :returns: tuple of strings with unescaped sequences
+    """
+    unescape_re = re.compile(r'\\{}'.format(seq))
+
+    return tuple(re.sub(unescape_re, seq, a) for a in args)
+
+
+def escape_seq(seq, *args):
+    """
+    escape (prepend '\\') all occurences of sequence in input strings
+
+    :param seq: sequence to escape
+    :param args: input string to process
+
+    :returns: tuple of strings with escaped sequences
+    """
+
+    return tuple(a.replace(seq, u'\\{}'.format(seq)) for a in args)
