@@ -606,7 +606,7 @@ def parse_log_level(level):
     if isinstance(level, six.string_types):
         try:
             level = int(level)
-        except:
+        except ValueError:
             pass
 
     # If it's a string lookup it's name and map to logging level
@@ -805,7 +805,7 @@ class LogManager(object):
         default_level attribute but that does not update the configure_state,
         this method is provided as a convenience to simultaneously set the
         configure_state if so desired.
-        
+
         :parameters:
           level
             The new default level for the log manager.  See
@@ -872,7 +872,7 @@ class LogManager(object):
           List of (regexp, level) tuples. This is a an ordered list
           regular expressions used to match against a logger name to
           configure the logger's level. The first regexp in the
-          sequence which matches the logger name will use the the
+          sequence which matches the logger name will use the
           level bound to that regexp to set the logger's level. If
           no regexp matches the logger name then the logger will be
           assigned the default_level.
