@@ -25,7 +25,6 @@ DOCLINES = __doc__.split("\n")
 
 import os
 import sys
-import distutils.sysconfig
 
 CLASSIFIERS = """\
 Development Status :: 4 - Beta
@@ -38,7 +37,8 @@ Operating System :: Unix
 
 # BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
 # update it when the contents of directories change.
-if os.path.exists('MANIFEST'): os.remove('MANIFEST')
+if os.path.exists('MANIFEST'):
+    os.remove('MANIFEST')
 
 def setup_package():
 
@@ -52,7 +52,7 @@ def setup_package():
     try:
         setup(
             name = "ipapython",
-            version = "4.3.2",
+            version = "4.4.3",
             license = "GPL",
             author = "Karl MacMillan, et.al.",
             author_email = "kmacmill@redhat.com",
