@@ -17,6 +17,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
 from __future__ import print_function
 
 import tempfile
@@ -27,23 +28,29 @@ import re
 import sys
 import time
 
+import dns.resolver
 import ldap
 import six
 
+from ipaserver.dns_data_management import (
+    IPASystemRecords,
+    IPADomainIsNotManagedByIPAError,
+)
 from ipaserver.install import installutils
 from ipaserver.install import service
 from ipaserver.install import sysupgrade
 from ipapython import sysrestore, ipautil, ipaldap
-from ipapython.ipa_log_manager import *
 from ipapython import dnsutil
+from ipapython.dnsutil import DNSName
+from ipapython.ipa_log_manager import root_logger
 from ipapython.dn import DN
+from ipapython.admintool import ScriptError
 import ipalib
 from ipalib import api, errors
 from ipalib.constants import IPA_CA_RECORD
 from ipaplatform import services
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
-from ipaplatform.tasks import tasks
 from ipalib.util import (validate_zonemgr_str, normalize_zonemgr,
                          get_dns_forward_zone_update_policy,
                          get_dns_reverse_zone_update_policy,
@@ -467,7 +474,7 @@ def check_reverse_zones(ip_addresses, reverse_zones, options, unattended,
             except ValueError as e:
                 msg = "Reverse zone %s will not be used: %s" % (rz, e)
                 if unattended:
-                    sys.exit(msg)
+                    raise ScriptError(msg)
                 else:
                     root_logger.warning(msg)
                 continue
@@ -534,6 +541,32 @@ def check_forwarders(dns_forwarders, logger):
     return forwarders_dnssec_valid
 
 
+def remove_master_dns_records(hostname, realm):
+    bind = BindInstance()
+    bind.remove_master_dns_records(hostname, realm, realm.lower())
+    bind.remove_server_ns_records(hostname)
+
+
+def ensure_dnsserver_container_exists(ldap, api_instance, logger=None):
+    """
+    Create cn=servers,cn=dns,$SUFFIX container. If logger is not None, emit a
+    message that the container already exists when DuplicateEntry is raised
+    """
+
+    entry = ldap.make_entry(
+        DN(api_instance.env.container_dnsservers, api_instance.env.basedn),
+        {
+            u'objectclass': [u'top', u'nsContainer'],
+            u'cn': [u'servers']
+        }
+    )
+    try:
+        ldap.add_entry(entry)
+    except errors.DuplicateEntry:
+        if logger is not None:
+            logger.debug('cn=servers,cn=dns container already exists')
+
+
 class DnsBackup(object):
     def __init__(self, service):
         self.service = service
@@ -574,7 +607,7 @@ class DnsBackup(object):
                     try:
                         delkw = { '%srecord' % str(type.lower()) : unicode(rdata) }
                         api.Command.dnsrecord_del(unicode(zone), unicode(host), **delkw)
-                    except:
+                    except Exception:
                         pass
                 j += 1
 
@@ -613,9 +646,9 @@ class BindInstance(service.Service):
     suffix = ipautil.dn_attribute_property('_suffix')
 
     def setup(self, fqdn, ip_addresses, realm_name, domain_name, forwarders,
-              forward_policy, ntp, reverse_zones,
+              forward_policy, reverse_zones,
               named_user=constants.NAMED_USER, zonemgr=None,
-              ca_configured=None, no_dnssec_validation=False):
+              no_dnssec_validation=False):
         self.named_user = named_user
         self.fqdn = fqdn
         self.ip_addresses = ip_addresses
@@ -625,9 +658,7 @@ class BindInstance(service.Service):
         self.forward_policy = forward_policy
         self.host = fqdn.split(".")[0]
         self.suffix = ipautil.realm_to_suffix(self.realm)
-        self.ntp = ntp
         self.reverse_zones = reverse_zones
-        self.ca_configured = ca_configured
         self.no_dnssec_validation=no_dnssec_validation
 
         if not zonemgr:
@@ -656,18 +687,23 @@ class BindInstance(service.Service):
     def host_in_default_domain(self):
         return normalize_zone(self.host_domain) == normalize_zone(self.domain)
 
-    def create_sample_bind_zone(self):
-        bind_txt = ipautil.template_file(ipautil.SHARE_DIR + "bind.zone.db.template", self.sub_dict)
-        [bind_fd, bind_name] = tempfile.mkstemp(".db","sample.zone.")
-        os.write(bind_fd, bind_txt)
-        os.close(bind_fd)
-        print("Sample zone file for bind has been created in "+bind_name)
+    def create_file_with_system_records(self):
+        system_records = IPASystemRecords(self.api)
+        text = u'\n'.join(
+            IPASystemRecords.records_list_from_zone(
+                system_records.get_base_records()
+            )
+        )
+        [fd, name] = tempfile.mkstemp(".db","ipa.system.records.")
+        os.write(fd, text)
+        os.close(fd)
+        print("Please add records in this file to your DNS system:", name)
 
     def create_instance(self):
 
         try:
             self.stop()
-        except:
+        except Exception:
             pass
 
         # get a connection to the DS
@@ -693,10 +729,11 @@ class BindInstance(service.Service):
             self.step("setting up records for other masters", self.__add_others)
         # all zones must be created before this step
         self.step("adding NS record to the zones", self.__add_self_ns)
-        self.step("setting up CA record", self.__add_ipa_ca_record)
 
         self.step("setting up kerberos principal", self.__setup_principal)
         self.step("setting up named.conf", self.__setup_named_conf)
+        self.step("setting up server configuration",
+            self.__setup_server_configuration)
 
         # named has to be started after softhsm initialization
         # self.step("restarting named", self.__start)
@@ -750,41 +787,10 @@ class BindInstance(service.Service):
             root_logger.debug("Unable to mask named (%s)", e)
 
     def __setup_sub_dict(self):
-        if self.forwarders:
-            fwds = "\n"
-            for forwarder in self.forwarders:
-                fwds += "\t\t%s;\n" % forwarder
-            fwds += "\t"
-        else:
-            fwds = " "
-
-        if self.ntp:
-            optional_ntp =  "\n;ntp server\n"
-            optional_ntp += "_ntp._udp\t\tIN SRV 0 100 123\t%s" % self.host_in_rr
-        else:
-            optional_ntp = ""
-
-        ipa_ca = ""
-        for addr in self.ip_addresses:
-            if addr.version in (4, 6):
-                ipa_ca += "%s\t\t\tIN %s\t\t\t%s\n" % (
-                    IPA_CA_RECORD,
-                    "A" if addr.version == 4 else "AAAA",
-                    str(addr))
-
         self.sub_dict = dict(
             FQDN=self.fqdn,
-            IP=[str(ip) for ip in self.ip_addresses],
-            DOMAIN=self.domain,
-            HOST=self.host,
-            REALM=self.realm,
             SERVER_ID=installutils.realm_to_serverid(self.realm),
-            FORWARDERS=fwds,
-            FORWARD_POLICY=self.forward_policy,
             SUFFIX=self.suffix,
-            OPTIONAL_NTP=optional_ntp,
-            ZONEMGR=self.zonemgr,
-            IPA_CA_RECORD=ipa_ca,
             BINDKEYS_FILE=paths.NAMED_BINDKEYS_FILE,
             MANAGED_KEYS_DIR=paths.NAMED_MANAGED_KEYS_DIR,
             ROOT_KEY=paths.NAMED_ROOT_KEY,
@@ -854,29 +860,6 @@ class BindInstance(service.Service):
     def __add_master_records(self, fqdn, addrs):
         host, zone = fqdn.split(".", 1)
 
-        if normalize_zone(zone) == normalize_zone(self.domain):
-            host_in_rr = host
-        else:
-            host_in_rr = normalize_zone(fqdn)
-
-        srv_records = (
-            ("_ldap._tcp", "0 100 389 %s" % host_in_rr),
-            ("_kerberos._tcp", "0 100 88 %s" % host_in_rr),
-            ("_kerberos._udp", "0 100 88 %s" % host_in_rr),
-            ("_kerberos-master._tcp", "0 100 88 %s" % host_in_rr),
-            ("_kerberos-master._udp", "0 100 88 %s" % host_in_rr),
-            ("_kpasswd._tcp", "0 100 464 %s" % host_in_rr),
-            ("_kpasswd._udp", "0 100 464 %s" % host_in_rr),
-        )
-        if self.ntp:
-            srv_records += (
-                ("_ntp._udp", "0 100 123 %s" % host_in_rr),
-            )
-
-        for (rname, rdata) in srv_records:
-            add_rr(self.domain, rname, "SRV", rdata, self.dns_backup,
-                   api=self.api)
-
         # Add forward and reverse records to self
         for addr in addrs:
             try:
@@ -907,63 +890,6 @@ class BindInstance(service.Service):
             root_logger.debug("Adding DNS records for master %s" % fqdn)
             self.__add_master_records(fqdn, addrs)
 
-    def __add_ipa_ca_records(self, fqdn, addrs, ca_configured):
-        if ca_configured is False:
-            root_logger.debug("CA is not configured")
-            return
-        elif ca_configured is None:
-            # we do not know if CA is configured for this host and we can
-            # add the CA record. So we need to find out
-            root_logger.debug("Check if CA is enabled for this host")
-            base_dn = DN(('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'),
-                         ('cn', 'etc'), self.api.env.basedn)
-            ldap_filter = '(&(objectClass=ipaConfigObject)(cn=CA))'
-            try:
-                self.api.Backend.ldap2.find_entries(filter=ldap_filter, base_dn=base_dn)
-            except ipalib.errors.NotFound:
-                root_logger.debug("CA is not configured")
-                return
-            else:
-                root_logger.debug("CA is configured for this host")
-
-        try:
-            for addr in addrs:
-                add_fwd_rr(self.domain, IPA_CA_RECORD, addr, api=self.api)
-        except errors.ValidationError:
-            # there is a CNAME record in ipa-ca, we can't add A/AAAA records
-            pass
-
-    def __add_ipa_ca_record(self):
-        self.__add_ipa_ca_records(self.fqdn, self.ip_addresses,
-                                  self.ca_configured)
-
-        if self.first_instance:
-            ldap = self.api.Backend.ldap2
-            try:
-                entries = ldap.get_entries(
-                    DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
-                       self.api.env.basedn),
-                    ldap.SCOPE_SUBTREE, '(&(objectClass=ipaConfigObject)(cn=CA))',
-                    ['dn'])
-            except errors.NotFound:
-                root_logger.debug('No server with CA found')
-                entries = []
-
-            for entry in entries:
-                fqdn = entry.dn[1]['cn']
-                if fqdn == self.fqdn:
-                    continue
-
-                host, zone = fqdn.split('.', 1)
-                if dns_zone_exists(zone, self.api):
-                    addrs = get_fwd_rr(zone, host, api=self.api)
-                else:
-                    addrs = dnsutil.resolve_ip_addresses(fqdn)
-                    # hack, will go away with locations
-                    addrs = [str(addr) for addr in addrs]
-
-                self.__add_ipa_ca_records(fqdn, addrs, True)
-
     def __setup_principal(self):
         dns_principal = "DNS/" + self.fqdn + "@" + self.realm
         installutils.kadmin_addprinc(dns_principal)
@@ -1036,6 +962,27 @@ class BindInstance(service.Service):
             'forward_policy_conflict_with_empty_zones_handled', True
         )
 
+    def __setup_server_configuration(self):
+        ensure_dnsserver_container_exists(self.admin_conn, self.api)
+        try:
+            self.api.Command.dnsserver_add(
+                self.fqdn, idnssoamname=DNSName(self.fqdn).make_absolute(),
+            )
+        except errors.DuplicateEntry:
+            # probably reinstallation of DNS
+            pass
+
+        try:
+            self.api.Command.dnsserver_mod(
+                self.fqdn,
+                idnsforwarders=[unicode(f) for f in self.forwarders],
+                idnsforwardpolicy=unicode(self.forward_policy)
+            )
+        except errors.EmptyModlist:
+            pass
+
+        sysupgrade.set_upgrade_state('dns', 'server_config_to_ldap', True)
+
     def __setup_resolv_conf(self):
         if not self.fstore.has_file(RESOLV_CONF):
             self.fstore.backup_file(RESOLV_CONF)
@@ -1059,51 +1006,40 @@ class BindInstance(service.Service):
             resolv_fd.close()
         except IOError as e:
             root_logger.error('Could not write to resolv.conf: %s', e)
+        else:
+            # python DNS might have global resolver cached in this variable
+            # we have to re-initialize it because resolv.conf has changed
+            dns.resolver.default_resolver = None
 
     def __generate_rndc_key(self):
         installutils.check_entropy()
         ipautil.run([paths.GENERATE_RNDC_KEY])
 
     def add_master_dns_records(self, fqdn, ip_addresses, realm_name, domain_name,
-                               reverse_zones, ntp=False, ca_configured=None):
+                               reverse_zones):
         self.fqdn = fqdn
         self.ip_addresses = ip_addresses
         self.realm = realm_name
         self.domain = domain_name
         self.host = fqdn.split(".")[0]
         self.suffix = ipautil.realm_to_suffix(self.realm)
-        self.ntp = ntp
         self.reverse_zones = reverse_zones
-        self.ca_configured = ca_configured
         self.first_instance = False
         self.zonemgr = 'hostmaster.%s' % self.domain
 
         self.__add_self()
-        self.__add_ipa_ca_record()
 
-    def add_ipa_ca_dns_records(self, fqdn, domain_name, ca_configured=True):
-        if not self.api.Backend.ldap2.isconnected():
-            self.api.Backend.ldap2.connect(autobind=True)
-
-        host, zone = fqdn.split(".", 1)
-        if dns_zone_exists(zone, self.api):
-            addrs = get_fwd_rr(zone, host, api=self.api)
-        else:
-            addrs = dnsutil.resolve_ip_addresses(fqdn)
-            # hack, will go away with locations
-            addrs = [str(addr) for addr in addrs]
-
-        self.domain = domain_name
-
-        self.__add_ipa_ca_records(fqdn, addrs, ca_configured)
-
-    def convert_ipa_ca_cnames(self, domain_name):
+    def remove_ipa_ca_cnames(self, domain_name):
         # get ipa-ca CNAMEs
-        cnames = get_rr(domain_name, IPA_CA_RECORD, "CNAME", api=self.api)
+        try:
+            cnames = get_rr(domain_name, IPA_CA_RECORD, "CNAME", api=self.api)
+        except errors.NotFound:
+            # zone does not exists
+            cnames = None
         if not cnames:
             return
 
-        root_logger.info('Converting IPA CA CNAME records to A/AAAA records')
+        root_logger.info('Removing IPA CA CNAME records')
 
         # create CNAME to FQDN mapping
         cname_fqdn = {}
@@ -1130,39 +1066,19 @@ class BindInstance(service.Service):
             fqdn = cname_fqdn[cname]
             if fqdn not in masters:
                 root_logger.warning(
-                    "Cannot convert IPA CA CNAME records to A/AAAA records, "
-                    "please convert them manually if necessary")
+                    "Cannot remove IPA CA CNAME please remove them manually "
+                    "if necessary")
                 return
 
         # delete all CNAMEs
         for cname in cnames:
             del_rr(domain_name, IPA_CA_RECORD, "CNAME", cname, api=self.api)
 
-        # add A/AAAA records
-        for cname in cnames:
-            fqdn = cname_fqdn[cname]
-            self.add_ipa_ca_dns_records(fqdn, domain_name, None)
-
     def remove_master_dns_records(self, fqdn, realm_name, domain_name):
         host, zone = fqdn.split(".", 1)
         self.host = host
         self.fqdn = fqdn
         self.domain = domain_name
-        suffix = ipautil.realm_to_suffix(realm_name)
-
-        resource_records = (
-            ("_ldap._tcp", "SRV", "0 100 389 %s" % self.host_in_rr),
-            ("_kerberos._tcp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kerberos._udp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kerberos-master._tcp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kerberos-master._udp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kpasswd._tcp", "SRV", "0 100 464 %s" % self.host_in_rr),
-            ("_kpasswd._udp", "SRV", "0 100 464 %s" % self.host_in_rr),
-            ("_ntp._udp", "SRV", "0 100 123 %s" % self.host_in_rr),
-        )
-
-        for (record, type, rdata) in resource_records:
-            del_rr(self.domain, record, type, rdata, api=self.api)
 
         areclist = get_fwd_rr(zone, host, api=self.api)
         for rdata in areclist:
@@ -1173,18 +1089,7 @@ class BindInstance(service.Service):
                 record = get_reverse_record_name(rzone, rdata)
                 del_rr(rzone, record, "PTR", normalize_zone(fqdn),
                        api=self.api)
-
-    def remove_ipa_ca_dns_records(self, fqdn, domain_name):
-        host, zone = fqdn.split(".", 1)
-        if dns_zone_exists(zone, self.api):
-            addrs = get_fwd_rr(zone, host, api=self.api)
-        else:
-            addrs = dnsutil.resolve_ip_addresses(fqdn)
-            # hack, will go away with locations
-            addrs = [str(addr) for addr in addrs]
-
-        for addr in addrs:
-            del_fwd_rr(domain_name, IPA_CA_RECORD, addr, api=self.api)
+        self.update_system_records()
 
     def remove_server_ns_records(self, fqdn):
         """
@@ -1218,14 +1123,40 @@ class BindInstance(service.Service):
                 root_logger.debug("record %s in zone %s", record, zone)
                 del_ns_rr(zone, record, ns_rdata, api=self.api)
 
+    def update_system_records(self):
+        self.print_msg("Updating DNS system records")
+        system_records = IPASystemRecords(self.api)
+        try:
+            (
+                (_ipa_rec, failed_ipa_rec),
+                (_loc_rec, failed_loc_rec)
+            ) = system_records.update_dns_records()
+        except IPADomainIsNotManagedByIPAError:
+            root_logger.error(
+                "IPA domain is not managed by IPA, please update records "
+                "manually")
+        else:
+            if failed_ipa_rec or failed_loc_rec:
+                root_logger.error("Update of following records failed:")
+                for attr in (failed_ipa_rec, failed_loc_rec):
+                    for rname, node, error in attr:
+                        for record in IPASystemRecords.records_list_from_node(
+                                rname, node
+                        ):
+                            root_logger.error("%s (%s)", record, error)
+
     def check_global_configuration(self):
         """
         Check global DNS configuration in LDAP server and inform user when it
         set and thus overrides his configured options in named.conf.
         """
         result = self.api.Command.dnsconfig_show()
-        global_conf_set = any(param in result['result'] for \
-                              param in self.api.Object['dnsconfig'].params)
+
+        global_conf_set = any(
+            param.name in result['result'] for param in
+            self.api.Object['dnsconfig'].params() if
+            u'virtual_attribute' not in param.flags
+        )
 
         if not global_conf_set:
             print("Global DNS configuration in LDAP server is empty")
@@ -1240,6 +1171,13 @@ class BindInstance(service.Service):
         self.api.Command.dnsconfig_show.output_for_cli(textui, result, None,
                                                        reverse=False)
 
+    def is_configured(self):
+        """
+        Override the default logic querying StateFile for configuration status
+        and look whether named.conf was already modified by IPA installer.
+        """
+        return named_conf_exists()
+
     def uninstall(self):
         if self.is_configured():
             self.print_msg("Unconfiguring %s" % self.service_name)
@@ -1257,7 +1195,6 @@ class BindInstance(service.Service):
                 self.fstore.restore_file(f)
             except ValueError as error:
                 root_logger.debug(error)
-                pass
 
         # disabled by default, by ldap_enable()
         if enabled:
