@@ -22,16 +22,16 @@ from __future__ import print_function
 
 import shutil
 import pwd
-import sys
 import os
 import re
 import time
 import tempfile
 import stat
+import fnmatch
 
 import ldap
 
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 from ipapython import ipautil, sysrestore, ipaldap
 from ipaserver.install import service
 from ipaserver.install import installutils
@@ -48,6 +48,7 @@ from ipaplatform.constants import constants as platformconstants
 from ipaplatform.tasks import tasks
 from ipalib.constants import CACERT
 from ipapython.dn import DN
+from ipapython.admintool import ScriptError
 from ipaplatform import services
 from ipaplatform.paths import paths
 
@@ -180,6 +181,16 @@ def get_domain_level(api=api):
     return int(entry.single_value['ipaDomainLevel'])
 
 
+def get_all_external_schema_files(root):
+    """Get all schema files"""
+    f = []
+    for path, subdirs, files in os.walk(root):
+        for name in files:
+            if fnmatch.fnmatch(name, "*.ldif"):
+                f.append(os.path.join(path, name))
+    return f
+
+
 INF_TEMPLATE = """
 [General]
 FullMachineName=   $FQDN
@@ -250,8 +261,7 @@ class DsInstance(service.Service):
 
         self.step("creating directory server user", create_ds_user)
         self.step("creating directory server instance", self.__create_instance)
-        if self.config_ldif:
-            self.step("updating configuration in dse.ldif", self.__update_dse_ldif)
+        self.step("updating configuration in dse.ldif", self.__update_dse_ldif)
         self.step("restarting directory server", self.__restart_instance)
         self.step("adding default schema", self.__add_default_schemas)
         self.step("enabling memberof plugin", self.__add_memberof_module)
@@ -351,7 +361,7 @@ class DsInstance(service.Service):
         self.start_creation(runtime=10)
 
     def create_replica(self, realm_name, master_fqdn, fqdn,
-                       domain_name, dm_password, subject_base,
+                       domain_name, dm_password, subject_base, api,
                        pkcs12_info=None, ca_file=None,
                        ca_is_configured=None, promote=False):
         # idstart and idmax are configured so that the range is seen as
@@ -376,6 +386,7 @@ class DsInstance(service.Service):
         if ca_is_configured is not None:
             self.ca_is_configured = ca_is_configured
         self.promote = promote
+        self.api = api
 
         self.__common_setup(enable_ssl=(not self.promote))
         self.step("restarting directory server", self.__restart_instance)
@@ -571,9 +582,15 @@ class DsInstance(service.Service):
             temp_filename = new_dse_ldif.name
             with open(dse_filename, "r") as input_file:
                 parser = installutils.ModifyLDIF(input_file, new_dse_ldif)
-                # parse modification from config ldif
-                with open(self.config_ldif, "r") as config_ldif:
-                    parser.modifications_from_ldif(config_ldif)
+                parser.replace_value(
+                        'cn=config,cn=ldbm database,cn=plugins,cn=config',
+                        'nsslapd-db-locks',
+                        ['50000']
+                        )
+                if self.config_ldif:
+                    # parse modifications from ldif file supplied by the admin
+                    with open(self.config_ldif, "r") as config_ldif:
+                        parser.modifications_from_ldif(config_ldif)
                 parser.parse()
             new_dse_ldif.flush()
         shutil.copy2(temp_filename, dse_filename)
@@ -614,7 +631,7 @@ class DsInstance(service.Service):
             super(DsInstance, self).restart(instance)
             if not is_ds_running(instance):
                 root_logger.critical("Failed to restart the directory server. See the installation log for details.")
-                sys.exit(1)
+                raise ScriptError()
         except SystemExit as e:
             raise e
         except Exception as e:
@@ -650,7 +667,9 @@ class DsInstance(service.Service):
         conn.unbind()
 
     def apply_updates(self):
-        data_upgrade = upgradeinstance.IPAUpgrade(self.realm)
+        schema_files = get_all_external_schema_files(paths.EXTERNAL_SCHEMA_DIR)
+        data_upgrade = upgradeinstance.IPAUpgrade(self.realm,
+                                                  schema_files=schema_files)
         try:
             data_upgrade.create_instance()
         except Exception as e:
@@ -923,7 +942,6 @@ class DsInstance(service.Service):
             self.fstore.restore_file(paths.SYSCONFIG_DIRSRV)
         except ValueError as error:
             root_logger.debug(error)
-            pass
 
         # disabled during IPA installation
         if enabled:
@@ -1216,7 +1234,8 @@ class DsInstance(service.Service):
         except OSError:
             pass
 
-        installutils.install_service_keytab(self.principal,
+        installutils.install_service_keytab(self.api,
+                                            self.principal,
                                             self.master_fqdn,
                                             paths.DS_KEYTAB,
                                             force_service_add=True)
