@@ -26,14 +26,13 @@
 from __future__ import print_function
 
 import os
-import sys
 
 import six
 
 from ipalib import api
-from ipapython import ipautil, admintool
+from ipapython import admintool
 from ipaplatform.paths import paths
-from ipaserver.install import installutils, dsinstance, schemaupdate
+from ipaserver.install import installutils, schemaupdate
 from ipaserver.install.ldapupdate import LDAPUpdate, UPDATES_DIR, BadSyntax
 from ipaserver.install.upgradeinstance import IPAUpgrade
 
@@ -82,8 +81,7 @@ class LDAPUpdater(admintool.AdminTool):
         try:
             installutils.check_server_configuration()
         except RuntimeError as e:
-            print(unicode(e))
-            sys.exit(1)
+            raise admintool.ScriptError(e)
 
     def setup_logging(self):
         super(LDAPUpdater, self).setup_logging(log_file_mode='a')
