@@ -236,6 +236,10 @@ class ReplicaPrepare(admintool.AdminTool):
         except errors.DatabaseError as e:
             raise admintool.ScriptError(e.desc)
 
+        if ca_enabled and not ipautil.file_exists(paths.CA_CS_CFG_PATH):
+            raise admintool.ScriptError(
+                "CA is not installed on this server. "
+                "ipa-replica-prepare must be run on an IPA server with CA.")
         if not ca_enabled and not options.http_cert_files:
             raise admintool.ScriptError(
                 "Cannot issue certificates: a CA is not installed. Use the "
@@ -303,7 +307,7 @@ class ReplicaPrepare(admintool.AdminTool):
             if options.http_pin is None:
                 options.http_pin = installutils.read_password(
                     "Enter Apache Server private key unlock",
-                    confirm=False, validate=False)
+                    confirm=False, validate=False, retry=False)
                 if options.http_pin is None:
                     raise admintool.ScriptError(
                         "Apache Server private key unlock password required")
@@ -317,7 +321,7 @@ class ReplicaPrepare(admintool.AdminTool):
             if options.dirsrv_pin is None:
                 options.dirsrv_pin = installutils.read_password(
                     "Enter Directory Server private key unlock",
-                    confirm=False, validate=False)
+                    confirm=False, validate=False, retry=False)
                 if options.dirsrv_pin is None:
                     raise admintool.ScriptError(
                         "Directory Server private key unlock password required")
@@ -331,7 +335,7 @@ class ReplicaPrepare(admintool.AdminTool):
             if options.pkinit_pin is None:
                 options.pkinit_pin = installutils.read_password(
                     "Enter Kerberos KDC private key unlock",
-                    confirm=False, validate=False)
+                    confirm=False, validate=False, retry=False)
                 if options.pkinit_pin is None:
                     raise admintool.ScriptError(
                         "Kerberos KDC private key unlock password required")
@@ -347,13 +351,6 @@ class ReplicaPrepare(admintool.AdminTool):
                 "Apache Server SSL certificate and Directory Server SSL "
                  "certificate are not signed by the same CA certificate")
 
-        if (not ipautil.file_exists(paths.CA_CS_CFG_PATH) and
-                options.dirsrv_pin is None):
-            self.log.info("If you installed IPA with your own certificates "
-                "using PKCS#12 files you must provide PKCS#12 files for any "
-                "replicas you create as well.")
-            raise admintool.ScriptError("The replica must be created on the "
-                "primary IPA server.")
 
     def run(self):
         options = self.options
@@ -546,7 +543,7 @@ class ReplicaPrepare(admintool.AdminTool):
             except exceptions:
                 return False
         except Exception as e:
-            self.log.warn('Exception while waiting for DNS record: %s: %s',
+            self.log.warning('Exception while waiting for DNS record: %s: %s',
                           type(e).__name__, e)
 
         return True
