@@ -18,17 +18,14 @@
 #
 
 import os
-import sys
 import shutil
 import tempfile
 import time
 import pwd
 import ldif
 import itertools
-import locale
 
 from six.moves.configparser import SafeConfigParser
-import six
 
 from ipalib import api, errors
 from ipalib.constants import FQDN
@@ -371,7 +368,7 @@ class Restore(admintool.AdminTool):
                 self.log.info('Stopping IPA services')
                 result = run(['ipactl', 'stop'], raiseonerr=False)
                 if result.returncode not in [0, 6]:
-                    self.log.warn('Stopping IPA failed: %s' % result.error_log)
+                    self.log.warning('Stopping IPA failed: %s' % result.error_log)
 
                 self.restore_selinux_booleans()
 
@@ -847,7 +844,7 @@ class Restore(admintool.AdminTool):
         services.knownservices.certmonger.restart()
 
     def init_api(self, **overrides):
-        api.bootstrap(in_server=False, context='restore', **overrides)
+        api.bootstrap(in_server=True, context='restore', **overrides)
         api.finalize()
 
         self.instances = [installutils.realm_to_serverid(api.env.realm)]
