@@ -21,9 +21,6 @@ from __future__ import absolute_import
 from __future__ import print_function
 
 import shutil
-import fileinput
-import re
-import sys
 import os
 import pwd
 import socket
@@ -34,17 +31,13 @@ from ipaserver.install import installutils
 from ipapython import sysrestore
 from ipapython import ipautil
 from ipapython import kernel_keyring
-from ipalib import errors
 from ipalib.constants import CACERT
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 from ipapython.dn import DN
 
 from ipaserver.install import replication
 from ipaserver.install import ldapupdate
 
-import pyasn1.codec.ber.decoder
-import struct
-
 from ipaserver.install import certs
 from distutils import version
 from ipaplatform.constants import constants
@@ -104,6 +97,7 @@ class KrbInstance(service.Service):
             krbextradata=service_entry['krbextradata'],
             krblastpwdchange=service_entry['krblastpwdchange'],
             krbprincipalname=service_entry['krbprincipalname'],
+            krbcanonicalname=service_entry['krbcanonicalname'],
             krbprincipalkey=service_entry['krbprincipalkey'],
             serverhostname=[self.fqdn.split('.',1)[0]],
             cn=[self.fqdn],
@@ -142,7 +136,7 @@ class KrbInstance(service.Service):
         self.backup_state("running", self.is_running())
         try:
             self.stop()
-        except:
+        except Exception:
             # It could have been not running
             pass
 
@@ -217,7 +211,7 @@ class KrbInstance(service.Service):
     def __start_instance(self):
         try:
             self.start()
-        except:
+        except Exception:
             root_logger.critical("krb5kdc service failed to start")
 
     def __setup_sub_dict(self):
@@ -398,7 +392,7 @@ class KrbInstance(service.Service):
 
         try:
             self.stop()
-        except:
+        except Exception:
             pass
 
         for f in [paths.KRB5KDC_KDC_CONF, paths.KRB5_CONF]:
@@ -406,7 +400,6 @@ class KrbInstance(service.Service):
                 self.fstore.restore_file(f)
             except ValueError as error:
                 root_logger.debug(error)
-                pass
 
         # disabled by default, by ldap_enable()
         if enabled:
