@@ -19,18 +19,38 @@
 #
 
 from ipaserver.install import service
+from ipaserver.install import sysupgrade
 from ipapython import sysrestore
 from ipapython import ipautil
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 
 NTPD_OPTS_VAR = constants.NTPD_OPTS_VAR
 NTPD_OPTS_QUOTE = constants.NTPD_OPTS_QUOTE
 
+NTP_EXPOSED_IN_LDAP = 'exposed_in_ldap'
+
+
+def ntp_ldap_enable(fqdn, base_dn, realm):
+    ntp = NTPInstance(realm=realm)
+    is_exposed_in_ldap = sysupgrade.get_upgrade_state(
+        'ntp', NTP_EXPOSED_IN_LDAP)
+
+    was_running = ntp.is_running()
+
+    if ntp.is_configured() and not is_exposed_in_ldap:
+        ntp.ldap_enable('NTP', fqdn, None, base_dn)
+        sysupgrade.set_upgrade_state('ntp', NTP_EXPOSED_IN_LDAP, True)
+
+        if was_running:
+            ntp.start()
+
+
 class NTPInstance(service.Service):
-    def __init__(self, fstore=None):
+    def __init__(self, realm=None, fstore=None):
         service.Service.__init__(self, "ntpd", service_desc="NTP daemon")
+        self.realm = realm
 
         if fstore:
             self.fstore = fstore
@@ -178,7 +198,6 @@ class NTPInstance(service.Service):
             self.fstore.restore_file(paths.NTP_CONF)
         except ValueError as error:
             root_logger.debug(error)
-            pass
 
         if enabled:
             self.enable()
