@@ -10,7 +10,7 @@ import ldap
 
 from ipaserver.install import service
 from ipaserver.install import installutils
-from ipapython.ipa_log_manager import *
+from ipapython.ipa_log_manager import root_logger
 from ipapython.dn import DN
 from ipapython import sysrestore, ipautil, ipaldap
 from ipaplatform.constants import constants
@@ -51,7 +51,7 @@ class ODSExporterInstance(service.Service):
 
         try:
             self.stop()
-        except:
+        except Exception:
             pass
 
         # get a connection to the DS
