@@ -5,6 +5,7 @@
 from __future__ import print_function
 
 import collections
+from distutils.version import LooseVersion
 import dns.exception as dnsexception
 import dns.name as dnsname
 import dns.resolver as dnsresolver
@@ -12,21 +13,24 @@ import dns.reversename as dnsreversename
 import os
 import shutil
 import socket
-import sys
 import tempfile
 
 import six
 
 from ipapython import ipaldap, ipautil, sysrestore
 from ipapython.dn import DN
-from ipapython.install import common, core
 from ipapython.install.common import step
 from ipapython.install.core import Knob
 from ipapython.ipa_log_manager import root_logger
+from ipapython.admintool import ScriptError
 from ipaplatform import services
 from ipaplatform.tasks import tasks
 from ipaplatform.paths import paths
-from ipalib import api, certstore, constants, create_api, errors, x509
+from ipalib import api, certstore, constants, create_api, errors, rpc, x509
+from ipalib.util import (
+    network_ip_address_warning,
+    broadcast_ip_address_warning,
+)
 import ipaclient.ipachangeconf
 import ipaclient.ntpconf
 from ipaserver.install import (
@@ -71,7 +75,7 @@ def make_pkcs12_info(directory, cert_name, password_name):
         return None
 
 
-def install_http_certs(config, fstore):
+def install_http_certs(config, fstore, remote_api):
 
     # Obtain keytab for the HTTP service
     fstore.backup_file(paths.IPA_KEYTAB)
@@ -81,7 +85,8 @@ def install_http_certs(config, fstore):
         pass
 
     principal = 'HTTP/%s@%s' % (config.host_name, config.realm_name)
-    installutils.install_service_keytab(principal,
+    installutils.install_service_keytab(remote_api,
+                                        principal,
                                         config.master_host_name,
                                         paths.IPA_KEYTAB,
                                         force_service_add=True)
@@ -94,8 +99,8 @@ def install_http_certs(config, fstore):
     # FIXME: need Signing-Cert too ?
 
 
-def install_replica_ds(config, options, ca_is_configured, promote=False,
-                       pkcs12_info=None):
+def install_replica_ds(config, options, ca_is_configured, remote_api,
+                       promote=False, pkcs12_info=None):
     dsinstance.check_ports()
 
     # if we have a pkcs12 file, create the cert db from
@@ -123,6 +128,7 @@ def install_replica_ds(config, options, ca_is_configured, promote=False,
         ca_is_configured=ca_is_configured,
         ca_file=ca_file,
         promote=promote,
+        api=remote_api,
     )
 
     return ds
@@ -155,8 +161,7 @@ def install_ca_cert(ldap, base_dn, realm, cafile):
 
         os.chmod(constants.CACERT, 0o444)
     except Exception as e:
-        print("error copying files: " + str(e))
-        sys.exit(1)
+        raise ScriptError("error copying files: " + str(e))
 
 
 def install_http(config, auto_redirect, ca_is_configured, promote=False,
@@ -209,9 +214,7 @@ def install_dns_records(config, options, remote_api):
                                         str(ip),
                                         config.realm_name,
                                         config.domain_name,
-                                        reverse_zone,
-                                        not options.no_ntp,
-                                        options.setup_ca)
+                                        reverse_zone)
     except errors.NotFound as e:
         root_logger.debug('Replica DNS records could not be added '
                           'on master: %s', str(e))
@@ -225,13 +228,13 @@ def install_dns_records(config, options, remote_api):
 def check_dirsrv():
     (ds_unsecure, ds_secure) = dsinstance.check_ports()
     if not ds_unsecure or not ds_secure:
-        print("IPA requires ports 389 and 636 for the Directory Server.")
-        print("These are currently in use:")
+        msg = ("IPA requires ports 389 and 636 for the Directory Server.\n"
+               "These are currently in use:\n")
         if not ds_unsecure:
-            print("\t389")
+            msg += "\t389\n"
         if not ds_secure:
-            print("\t636")
-        sys.exit(1)
+            msg += "\t636\n"
+        raise ScriptError(msg)
 
 
 def check_dns_resolution(host_name, dns_servers):
@@ -324,26 +327,13 @@ def check_dns_resolution(host_name, dns_servers):
     return no_errors
 
 
-def check_ca_enabled(api):
-    try:
-        api.Backend.rpcclient.connect()
-        result = api.Backend.rpcclient.forward(
-            'ca_is_enabled',
-            version=u'2.112'    # All the way back to 3.0 servers
-        )
-        return result['result']
-    finally:
-        if api.Backend.rpcclient.isconnected():
-            api.Backend.rpcclient.disconnect()
-
-
 def configure_certmonger():
     messagebus = services.knownservices.messagebus
     try:
         messagebus.start()
     except Exception as e:
-        print("Messagebus service unavailable: %s" % str(e))
-        sys.exit(3)
+        raise ScriptError("Messagebus service unavailable: %s" % str(e),
+                          rval=3)
 
     # Ensure that certmonger has been started at least once to generate the
     # cas files in /var/lib/certmonger/cas.
@@ -351,14 +341,14 @@ def configure_certmonger():
     try:
         cmonger.restart()
     except Exception as e:
-        print("Certmonger service unavailable: %s" % str(e))
-        sys.exit(3)
+        raise ScriptError("Certmonger service unavailable: %s" % str(e),
+                          rval=3)
 
     try:
         cmonger.enable()
     except Exception as e:
-        print("Failed to enable Certmonger: %s" % str(e))
-        sys.exit(3)
+        raise ScriptError("Failed to enable Certmonger: %s" % str(e),
+                          rval=3)
 
 
 def remove_replica_info_dir(installer):
@@ -379,7 +369,7 @@ def common_cleanup(func):
                 remove_replica_info_dir(installer)
                 raise
         except KeyboardInterrupt:
-            sys.exit(1)
+            raise ScriptError()
         except Exception:
             print(
                 "Your system may be partly configured.\n"
@@ -492,6 +482,24 @@ def promote_openldap_conf(hostname, master):
         root_logger.info("Failed to update {}: {}".format(ldap_conf, e))
 
 
+def check_remote_version(api):
+    client = rpc.jsonclient(api)
+    client.finalize()
+
+    client.connect()
+    try:
+        env = client.forward(u'env', u'version')['result']
+    finally:
+        client.disconnect()
+
+    remote_version = env['version']
+    version = api.env.version
+    if LooseVersion(remote_version) > LooseVersion(version):
+        raise RuntimeError(
+            "Cannot install replica of a server of higher version ({}) than"
+            "the local version ({})".format(remote_version, version))
+
+
 @common_cleanup
 def install_check(installer):
     options = installer
@@ -504,15 +512,17 @@ def install_check(installer):
     tasks.check_selinux_status()
 
     if is_ipa_configured():
-        sys.exit("IPA server is already configured on this system.\n"
-                 "If you want to reinstall the IPA server, please uninstall "
-                 "it first using 'ipa-server-install --uninstall'.")
+        raise ScriptError(
+            "IPA server is already configured on this system.\n"
+            "If you want to reinstall the IPA server, please uninstall "
+            "it first using 'ipa-server-install --uninstall'.")
 
     client_fstore = sysrestore.FileStore(paths.IPA_CLIENT_SYSRESTORE)
     if client_fstore.has_files():
-        sys.exit("IPA client is already configured on this system.\n"
-                 "Please uninstall it first before configuring the replica, "
-                 "using 'ipa-client-install --uninstall'.")
+        raise ScriptError(
+            "IPA client is already configured on this system.\n"
+            "Please uninstall it first before configuring the replica, "
+            "using 'ipa-client-install --uninstall'.")
 
     sstore = sysrestore.StateFile(paths.SYSRESTORE)
 
@@ -520,7 +530,7 @@ def install_check(installer):
 
     # Check to see if httpd is already configured to listen on 443
     if httpinstance.httpd_443_configured():
-        sys.exit("Aborting installation")
+        raise ScriptError("Aborting installation")
 
     check_dirsrv()
 
@@ -541,9 +551,9 @@ def install_check(installer):
         try:
             dirman_password = get_dirman_password()
         except KeyboardInterrupt:
-            sys.exit(0)
+            raise ScriptError(rval=0)
         if dirman_password is None:
-            sys.exit("Directory Manager password required")
+            raise ScriptError("Directory Manager password required")
 
     config = create_replica_config(dirman_password, filename, options)
     installer._top_dir = config.top_dir
@@ -639,12 +649,12 @@ def install_check(installer):
         if replman.get_replication_agreement(config.host_name):
             root_logger.info('Error: A replication agreement for this '
                              'host already exists.')
-            print('A replication agreement for this host already exists. '
-                  'It needs to be removed.')
-            print("Run this on the master that generated the info file:")
-            print(("    %% ipa-replica-manage del %s --force" %
-                  config.host_name))
-            sys.exit(3)
+            msg = ("A replication agreement for this host already exists. "
+                   "It needs to be removed.\n"
+                   "Run this on the master that generated the info file:\n"
+                   "    %% ipa-replica-manage del %s --force" %
+                   config.host_name)
+            raise ScriptError(msg, rval=3)
 
         # Detect the current domain level
         try:
@@ -675,8 +685,7 @@ def install_check(installer):
                        "this version is allowed to be installed "
                        "within this domain.")
             root_logger.error(message)
-            print(message)
-            sys.exit(3)
+            raise ScriptError(message, rval=3)
 
         # Check pre-existing host entry
         try:
@@ -688,11 +697,11 @@ def install_check(installer):
         else:
             root_logger.info('Error: Host %s already exists on the master '
                              'server.' % config.host_name)
-            print(('The host %s already exists on the master server.' %
-                  config.host_name))
-            print("You should remove it before proceeding:")
-            print("    %% ipa host-del %s" % config.host_name)
-            sys.exit(3)
+            msg = ("The host %s already exists on the master server.\n"
+                   "You should remove it before proceeding:\n"
+                   "    %% ipa host-del %s" %
+                   (config.host_name, config.host_name))
+            raise ScriptError(msg, rval=3)
 
         dns_masters = remote_api.Object['dnsrecord'].get_dns_masters()
         if dns_masters:
@@ -704,7 +713,7 @@ def install_check(installer):
                     check_dns_resolution(config.host_name, dns_masters))
                 if not resolution_ok and installer.interactive:
                     if not ipautil.user_input("Continue?", False):
-                        sys.exit(0)
+                        raise ScriptError(rval=0)
         else:
             root_logger.debug('No IPA DNS servers, '
                               'skipping forward/reverse resolution check')
@@ -719,8 +728,7 @@ def install_check(installer):
             try:
                 kra.install_check(remote_api, config, options)
             except RuntimeError as e:
-                print(str(e))
-                sys.exit(1)
+                raise ScriptError(e)
 
         if options.setup_dns:
             dns.install_check(False, remote_api, True, options,
@@ -731,12 +739,16 @@ def install_check(installer):
                 config.host_name, not installer.interactive, False,
                 options.ip_addresses)
 
+            # check addresses here, dns module is doing own check
+            network_ip_address_warning(config.ips)
+            broadcast_ip_address_warning(config.ips)
+
     except errors.ACIError:
-        sys.exit("\nThe password provided is incorrect for LDAP server "
-                 "%s" % config.master_host_name)
+        raise ScriptError("\nThe password provided is incorrect for LDAP server "
+                          "%s" % config.master_host_name)
     except errors.LDAPError:
-        sys.exit("\nUnable to connect to LDAP server %s" %
-                 config.master_host_name)
+        raise ScriptError("\nUnable to connect to LDAP server %s" %
+                          config.master_host_name)
     finally:
         if replman and replman.conn:
             replman.conn.unbind()
@@ -794,7 +806,9 @@ def install(installer):
             ntp.create_instance()
 
         # Configure dirsrv
-        ds = install_replica_ds(config, options, ca_enabled)
+        ds = install_replica_ds(config, options, ca_enabled, remote_api)
+
+        ntpinstance.ntp_ldap_enable(config.host_name, ds.suffix, api.env.realm)
 
         # Always try to install DNS records
         install_dns_records(config, options, remote_api)
@@ -812,12 +826,17 @@ def install(installer):
         if ca_enabled:
             options.ra_p12 = config.dir + "/ra.p12"
 
-        ca.install(False, config, options)
+        ca.install_step_0(False, config, options)
 
     krb = install_krb(config, setup_pkinit=not options.no_pkinit)
     http = install_http(config, auto_redirect=not options.no_ui_redirect,
                         ca_is_configured=ca_enabled)
 
+    if config.setup_ca:
+        # Done after install_krb() because lightweight CA key
+        # retrieval setup needs to create kerberos principal.
+        ca.install_step_1(False, config, options)
+
     otpd = otpdinstance.OtpdInstance()
     otpd.create_instance('OTPD', config.host_name, config.dirman_password,
                          ipautil.realm_to_suffix(config.realm_name))
@@ -854,9 +873,11 @@ def install(installer):
     if config.setup_ca:
         services.knownservices['pki_tomcatd'].restart('pki-tomcat')
 
+    api.Backend.ldap2.connect(autobind=True)
     if options.setup_dns:
-        api.Backend.ldap2.connect(autobind=True)
         dns.install(False, True, options)
+    else:
+        api.Command.dns_update_system_records()
 
     # Restart httpd to pick up the new IPA configuration
     service.print_msg("Restarting the web server")
@@ -867,7 +888,7 @@ def install(installer):
     try:
         args = [paths.IPA_CLIENT_INSTALL, "--on-master", "--unattended",
                 "--domain", config.domain_name, "--server", config.host_name,
-                "--realm", config.realm_name]
+                "--realm", config.realm_name, "--no-ntp"]
         if options.no_dns_sshfp:
             args.append("--no-dns-sshfp")
         if options.ssh_trust_dns:
@@ -904,7 +925,7 @@ def ensure_enrolled(installer):
     try:
         installer._enrollment_performed = True
 
-        args = [paths.IPA_CLIENT_INSTALL, "--unattended"]
+        args = [paths.IPA_CLIENT_INSTALL, "--unattended", "--no-ntp"]
         stdin = None
 
         if installer.domain_name:
@@ -941,7 +962,7 @@ def ensure_enrolled(installer):
         ipautil.run(args, stdin=stdin, redirect_output=True)
         print()
     except Exception:
-        sys.exit("Configuration of client side components failed!")
+        raise ScriptError("Configuration of client side components failed!")
 
 
 def promotion_check_ipa_domain(master_ldap_conn, basedn):
@@ -981,9 +1002,10 @@ def promote_check(installer):
     tasks.check_selinux_status()
 
     if is_ipa_configured():
-        sys.exit("IPA server is already configured on this system.\n"
-                 "If you want to reinstall the IPA server, please uninstall "
-                 "it first using 'ipa-server-install --uninstall'.")
+        raise ScriptError(
+            "IPA server is already configured on this system.\n"
+            "If you want to reinstall the IPA server, please uninstall "
+            "it first using 'ipa-server-install --uninstall'.")
 
     client_fstore = sysrestore.FileStore(paths.IPA_CLIENT_SYSRESTORE)
     if not client_fstore.has_files():
@@ -1001,7 +1023,7 @@ def promote_check(installer):
 
     # Check to see if httpd is already configured to listen on 443
     if httpinstance.httpd_443_configured():
-        sys.exit("Aborting installation")
+        raise ScriptError("Aborting installation")
 
     check_dirsrv()
 
@@ -1016,7 +1038,7 @@ def promote_check(installer):
         except ipaclient.ntpconf.NTPConfigurationError:
             pass
 
-    api.bootstrap(context='installer')
+    api.bootstrap(in_server=True, context='installer')
     api.finalize()
 
     config = ReplicaConfig()
@@ -1042,7 +1064,7 @@ def promote_check(installer):
                 "Enter Apache Server private key unlock",
                 confirm=False, validate=False)
             if options.http_pin is None:
-                sys.exit(
+                raise ScriptError(
                     "Apache Server private key unlock password required")
         http_pkcs12_file, http_pin, http_ca_cert = load_pkcs12(
             cert_files=options.http_cert_files,
@@ -1058,7 +1080,7 @@ def promote_check(installer):
                 "Enter Directory Server private key unlock",
                 confirm=False, validate=False)
             if options.dirsrv_pin is None:
-                sys.exit(
+                raise ScriptError(
                     "Directory Server private key unlock password required")
         dirsrv_pkcs12_file, dirsrv_pin, dirsrv_ca_cert = load_pkcs12(
             cert_files=options.dirsrv_cert_files,
@@ -1074,7 +1096,7 @@ def promote_check(installer):
                 "Enter Kerberos KDC private key unlock",
                 confirm=False, validate=False)
             if options.pkinit_pin is None:
-                sys.exit(
+                raise ScriptError(
                     "Kerberos KDC private key unlock password required")
         pkinit_pkcs12_file, pkinit_pin, pkinit_ca_cert = load_pkcs12(
             cert_files=options.pkinit_cert_files,
@@ -1104,10 +1126,15 @@ def promote_check(installer):
                            "the client and try again.")
 
     ldapuri = 'ldaps://%s' % ipautil.format_netloc(config.master_host_name)
+    xmlrpc_uri = 'https://{}/ipa/xml'.format(
+        ipautil.format_netloc(config.master_host_name))
     remote_api = create_api(mode=None)
     remote_api.bootstrap(in_server=True, context='installer',
-                         ldap_uri=ldapuri)
+                         ldap_uri=ldapuri, xmlrpc_uri=xmlrpc_uri)
     remote_api.finalize()
+
+    check_remote_version(remote_api)
+
     conn = remote_api.Backend.ldap2
     replman = None
     try:
@@ -1184,7 +1211,7 @@ def promote_check(installer):
             print("Run this command:")
             print("    %% ipa-replica-manage del %s --force" %
                   config.host_name)
-            sys.exit(3)
+            raise ScriptError(rval=3)
 
         # Detect if current level is out of supported range
         # for this IPA version
@@ -1199,7 +1226,7 @@ def promote_check(installer):
                        "this version is allowed to be installed "
                        "within this domain.")
             root_logger.error(message)
-            sys.exit(3)
+            raise ScriptError(rval=3)
 
         # Detect if the other master can handle replication managers
         # cn=replication managers,cn=sysaccounts,cn=etc,$SUFFIX
@@ -1215,7 +1242,7 @@ def promote_check(installer):
                    "command on the master and use a prep file to install "
                    "this replica.")
             root_logger.error(msg)
-            sys.exit(3)
+            raise ScriptError(rval=3)
 
         dns_masters = remote_api.Object['dnsrecord'].get_dns_masters()
         if dns_masters:
@@ -1227,7 +1254,7 @@ def promote_check(installer):
                     check_dns_resolution(config.host_name, dns_masters))
                 if not resolution_ok and installer.interactive:
                     if not ipautil.user_input("Continue?", False):
-                        sys.exit(0)
+                        raise ScriptError(rval=0)
         else:
             root_logger.debug('No IPA DNS servers, '
                               'skipping forward/reverse resolution check')
@@ -1245,7 +1272,7 @@ def promote_check(installer):
             if options.dirsrv_cert_files:
                 root_logger.error("Certificates could not be provided when "
                                   "CA is present on some master.")
-                sys.exit(3)
+                raise ScriptError(rval=3)
         else:
             ca_enabled = False
             if not options.dirsrv_cert_files:
@@ -1253,20 +1280,20 @@ def promote_check(installer):
                                   "installed. Use the --http-cert-file, "
                                   "--dirsrv-cert-file options to provide "
                                   "custom certificates.")
-                sys.exit(3)
+                raise ScriptError(rval=3)
 
         config.kra_host_name = service.find_providing_server('KRA', conn,
                                                              api.env.server)
         if options.setup_kra and config.kra_host_name is None:
             root_logger.error("There is no KRA server in the domain, can't "
                               "setup a KRA clone")
-            sys.exit(3)
+            raise ScriptError(rval=3)
 
         if options.setup_ca:
             if not ca_enabled:
                 root_logger.error("The remote master does not have a CA "
                                   "installed, can't set up CA")
-                sys.exit(3)
+                raise ScriptError(rval=3)
 
             options.realm_name = config.realm_name
             options.host_name = config.host_name
@@ -1277,22 +1304,26 @@ def promote_check(installer):
             try:
                 kra.install_check(remote_api, config, options)
             except RuntimeError as e:
-                print(str(e))
-                sys.exit(1)
+                raise ScriptError(e)
 
         if options.setup_dns:
             dns.install_check(False, remote_api, True, options,
                               config.host_name)
+            config.ips = dns.ip_addresses
         else:
             config.ips = installutils.get_server_ip_address(
                 config.host_name, not installer.interactive,
                 False, options.ip_addresses)
 
+            # check addresses here, dns module is doing own check
+            network_ip_address_warning(config.ips)
+            broadcast_ip_address_warning(config.ips)
+
     except errors.ACIError:
-        sys.exit("\nInsufficient privileges to promote the server.")
+        raise ScriptError("\nInsufficient privileges to promote the server.")
     except errors.LDAPError:
-        sys.exit("\nUnable to connect to LDAP server %s" %
-                 config.master_host_name)
+        raise ScriptError("\nUnable to connect to LDAP server %s" %
+                          config.master_host_name)
     finally:
         if replman and replman.conn:
             replman.conn.unbind()
@@ -1348,10 +1379,10 @@ def promote(installer):
     pkinit_pkcs12_file = installer._pkinit_pkcs12_file
     pkinit_pkcs12_info = installer._pkinit_pkcs12_info
 
+    ccache = os.environ['KRB5CCNAME']
+    remote_api = installer._remote_api
+    conn = remote_api.Backend.ldap2
     if installer._add_to_ipaservers:
-        ccache = os.environ['KRB5CCNAME']
-        remote_api = installer._remote_api
-        conn = remote_api.Backend.ldap2
         try:
             conn.connect(ccache=installer._ccache)
 
@@ -1388,18 +1419,27 @@ def promote(installer):
         ntp.create_instance()
 
     try:
+        conn.connect(ccache=ccache)
+
         # Configure dirsrv
         ds = install_replica_ds(config, options, installer._ca_enabled,
+                                remote_api,
                                 promote=True, pkcs12_info=dirsrv_pkcs12_info)
 
         # Always try to install DNS records
-        install_dns_records(config, options, api)
+        install_dns_records(config, options, remote_api)
 
         # Must install http certs before changing ipa configuration file
         # or certmonger will fail to contact the peer master
-        install_http_certs(config, fstore)
+        install_http_certs(config, fstore, remote_api)
+
+        ntpinstance.ntp_ldap_enable(config.host_name, ds.suffix,
+                                    remote_api.env.realm)
 
     finally:
+        if conn.isconnected():
+            conn.disconnect()
+
         # Create the management framework config file
         # do this regardless of the state of DS installation. Even if it fails,
         # we need to have master-like configuration in order to perform a
@@ -1477,8 +1517,7 @@ def promote(installer):
 
         ca = cainstance.CAInstance(config.realm_name, certs.NSS_DIR,
                                    host_name=config.host_name,
-                                   dm_password=config.dirman_password,
-                                   api=remote_api)
+                                   dm_password=config.dirman_password)
         ca.configure_replica(config.ca_host_name,
                              subject_base=config.subject_base,
                              ca_cert_bundle=ca_data)
@@ -1510,10 +1549,11 @@ def promote(installer):
     server_api.bootstrap(in_server=True, context='installer')
     server_api.finalize()
 
+    server_api.Backend.ldap2.connect(autobind=True)
     if options.setup_dns:
-        server_api.Backend.rpcclient.connect()
-        server_api.Backend.ldap2.connect(autobind=True)
         dns.install(False, True, options, server_api)
+    else:
+        server_api.Command.dns_update_system_records()
 
     # Everything installed properly, activate ipa service.
     services.knownservices.ipa.enable()
