@@ -1,7 +1,7 @@
 # Authors:
 #   Jason Gerard DeRose <jderose@redhat.com>
 #
-# Copyright (C) 2008  Red Hat
+# Copyright (C) 2008-2016  Red Hat
 # see file 'COPYING' for use and warranty information
 #
 # This program is free software; you can redistribute it and/or modify
@@ -40,18 +40,19 @@ from six.moves.urllib.parse import parse_qs
 
 from ipalib import plugable, errors
 from ipalib.capabilities import VERSION_WITHOUT_CAPABILITIES
+from ipalib.frontend import Local
 from ipalib.backend import Executioner
-from ipalib.errors import (PublicError, InternalError, CommandError, JSONError,
+from ipalib.errors import (PublicError, InternalError, JSONError,
     CCacheError, RefererError, InvalidSessionPassword, NotFound, ACIError,
-    ExecutionError, PasswordExpired)
+    ExecutionError, PasswordExpired, KrbPrincipalExpired, UserLocked)
 from ipalib.request import context, destroy_context
 from ipalib.rpc import (xml_dumps, xml_loads,
     json_encode_binary, json_decode_binary)
 from ipalib.util import parse_time_duration, normalize_name
 from ipapython.dn import DN
 from ipaserver.plugins.ldap2 import ldap2
-from ipalib.session import (
-    session_mgr, AuthManager, get_ipa_ccache_name,
+from ipaserver.session import (
+    get_session_mgr, AuthManager, get_ipa_ccache_name,
     load_ccache_data, bind_ipa_ccache, release_ipa_ccache, fmt_time,
     default_max_session_duration, krbccache_dir, krbccache_prefix)
 from ipalib.backend import Backend
@@ -310,6 +311,21 @@ class WSGIExecutioner(Executioner):
         if 'wsgi_dispatch' in self.api.Backend:
             self.api.Backend.wsgi_dispatch.mount(self, self.key)
 
+    def _get_command(self, name):
+        try:
+            # assume version 1 for unversioned command calls
+            command = self.api.Command[name, '1']
+        except KeyError:
+            try:
+                command = self.api.Command[name]
+            except KeyError:
+                command = None
+
+        if command is None or isinstance(command, Local):
+            raise errors.CommandError(name=name)
+
+        return command
+
     def wsgi_execute(self, environ):
         result = None
         error = None
@@ -318,6 +334,7 @@ class WSGIExecutioner(Executioner):
         name = None
         args = ()
         options = {}
+        command = None
 
         e = None
         if not 'HTTP_REFERER' in environ:
@@ -330,7 +347,7 @@ class WSGIExecutioner(Executioner):
                 lang_reg = lang_reg_w_q.split(';')[0]
                 lang_ = lang_reg.split('-')[0]
                 if '-' in lang_reg:
-                    reg = lang_reg.split('-')[1].upper();
+                    reg = lang_reg.split('-')[1].upper()
                 else:
                     reg = lang_.upper()
                 os.environ['LANG'] = '%s_%s' % (lang_, reg)
@@ -344,10 +361,9 @@ class WSGIExecutioner(Executioner):
                 (name, args, options, _id) = self.simple_unmarshal(environ)
             if name in self._system_commands:
                 result = self._system_commands[name](self, *args, **options)
-            elif name not in self.Command:
-                raise CommandError(name=name)
             else:
-                result = self.Command[name](*args, **options)
+                command = self._get_command(name)
+                result = command(*args, **options)
         except PublicError as e:
             if self.api.env.debug:
                 self.debug('WSGI wsgi_execute PublicError: %s', traceback.format_exc())
@@ -361,9 +377,9 @@ class WSGIExecutioner(Executioner):
             os.environ['LANG'] = lang
 
         principal = getattr(context, 'principal', 'UNKNOWN')
-        if name and name in self.Command:
+        if command is not None:
             try:
-                params = self.Command[name].args_options_2_params(*args, **options)
+                params = command.args_options_2_params(*args, **options)
             except Exception as e:
                 self.info(
                    'exception %s caught when converting options: %s', e.__class__.__name__, str(e)
@@ -378,7 +394,7 @@ class WSGIExecutioner(Executioner):
                       type(self).__name__,
                       principal,
                       name,
-                      ', '.join(self.Command[name]._repr_iter(**params)),
+                      ', '.join(command._repr_iter(**params)),
                       result_string)
         else:
             self.info('[%s] %s: %s: %s',
@@ -415,6 +431,7 @@ class WSGIExecutioner(Executioner):
         if session_data is not None:
             # Send session cookie back and store session data
             # FIXME: the URL path should be retreived from somewhere (but where?), not hardcoded
+            session_mgr = get_session_mgr()
             session_cookie = session_mgr.generate_cookie('/ipa', session_data['session_id'],
                                                          session_data['session_expiration_timestamp'])
             headers.append(('Set-Cookie', session_cookie))
@@ -423,11 +440,11 @@ class WSGIExecutioner(Executioner):
         return [response]
 
     def unmarshal(self, data):
-        raise NotImplementedError('%s.unmarshal()' % self.fullname)
+        raise NotImplementedError('%s.unmarshal()' % type(self).__name__)
 
     def marshal(self, result, error, _id=None,
                 version=VERSION_WITHOUT_CAPABILITIES):
-        raise NotImplementedError('%s.marshal()' % self.fullname)
+        raise NotImplementedError('%s.marshal()' % type(self).__name__)
 
 
 class jsonserver(WSGIExecutioner, HTTP_Status):
@@ -457,6 +474,7 @@ class jsonserver(WSGIExecutioner, HTTP_Status):
             error = dict(
                 code=error.errno,
                 message=error.strerror,
+                data=error.kw,
                 name=unicode(error.__class__.__name__),
             )
         principal = getattr(context, 'principal', 'UNKNOWN')
@@ -575,6 +593,7 @@ class KerberosSession(object):
         krb_expiration = krb_endtime - krb_ticket_expiration_threshold
 
         # Set the session expiration time
+        session_mgr = get_session_mgr()
         session_mgr.set_session_expiration_time(session_data,
                                                 duration=self.session_auth_duration,
                                                 max_age=krb_expiration,
@@ -586,6 +605,7 @@ class KerberosSession(object):
             headers = []
 
         # Retrieve the session data (or newly create)
+        session_mgr = get_session_mgr()
         session_data = session_mgr.load_session_data(environ.get('HTTP_COOKIE'))
         session_id = session_data['session_id']
 
@@ -672,7 +692,8 @@ class xmlserver(KerberosWSGIExecutioner):
         """list methods for XML-RPC introspection"""
         if params:
             raise errors.ZeroArgumentError(name='system.listMethods')
-        return (tuple(unicode(name) for name in self.Command) +
+        return (tuple(unicode(cmd.name) for cmd in self.Command()
+                      if cmd is self.Command[cmd.name]) +
                 tuple(unicode(name) for name in self._system_commands))
 
     def _get_method_name(self, name, *params):
@@ -691,22 +712,21 @@ class xmlserver(KerberosWSGIExecutioner):
             # TODO
             # for now let's not go out of our way to document standard XML-RPC
             return u'undef'
-        elif method_name in self.Command:
+        else:
+            self._get_command(method_name)
+
             # All IPA commands return a dict (struct),
             # and take a params, options - list and dict (array, struct)
             return [[u'struct', u'array', u'struct']]
-        else:
-            raise errors.CommandError(name=method_name)
 
     def methodHelp(self, *params):
         """get method docstring for XML-RPC introspection"""
         method_name = self._get_method_name('system.methodHelp', *params)
         if method_name in self._system_commands:
             return u''
-        elif method_name in self.Command:
-            return unicode(self.Command[method_name].__doc__ or '')
         else:
-            raise errors.CommandError(name=method_name)
+            command = self._get_command(method_name)
+            return unicode(command.doc or '')
 
     _system_commands = {
         'system.listMethods': listMethods,
@@ -750,6 +770,7 @@ class jsonserver_session(jsonserver, KerberosSession):
         super(jsonserver_session, self).__init__(api)
         name = '{0}_{1}'.format(self.__class__.__name__, id(self))
         auth_mgr = AuthManagerKerb(name)
+        session_mgr = get_session_mgr()
         session_mgr.auth_mgr.register(auth_mgr.name, auth_mgr)
 
     def _on_finalize(self):
@@ -773,6 +794,7 @@ class jsonserver_session(jsonserver, KerberosSession):
         self.debug('WSGI jsonserver_session.__call__:')
 
         # Load the session data
+        session_mgr = get_session_mgr()
         session_data = session_mgr.load_session_data(environ.get('HTTP_COOKIE'))
         session_id = session_data['session_id']
 
@@ -846,16 +868,16 @@ class jsonserver_kerb(jsonserver, KerberosWSGIExecutioner):
     key = '/json'
 
 
-class login_kerberos(Backend, KerberosSession, HTTP_Status):
-    key = '/session/login_kerberos'
+class KerberosLogin(Backend, KerberosSession, HTTP_Status):
+    key = None
 
     def _on_finalize(self):
-        super(login_kerberos, self)._on_finalize()
+        super(KerberosLogin, self)._on_finalize()
         self.api.Backend.wsgi_dispatch.mount(self, self.key)
         self.kerb_session_on_finalize()
 
     def __call__(self, environ, start_response):
-        self.debug('WSGI login_kerberos.__call__:')
+        self.debug('WSGI KerberosLogin.__call__:')
 
         # Get the ccache created by mod_auth_gssapi
         user_ccache_name=environ.get('KRB5CCNAME')
@@ -865,6 +887,15 @@ class login_kerberos(Backend, KerberosSession, HTTP_Status):
 
         return self.finalize_kerberos_acquisition('login_kerberos', user_ccache_name, environ, start_response)
 
+
+class login_kerberos(KerberosLogin):
+    key = '/session/login_kerberos'
+
+
+class login_x509(KerberosLogin):
+    key = '/session/login_x509'
+
+
 class login_password(Backend, KerberosSession, HTTP_Status):
 
     content_type = 'text/plain'
@@ -949,6 +980,16 @@ class login_password(Backend, KerberosSession, HTTP_Status):
             return self.unauthorized(environ, start_response, str(e), 'password-expired')
         except InvalidSessionPassword as e:
             return self.unauthorized(environ, start_response, str(e), 'invalid-password')
+        except KrbPrincipalExpired as e:
+            return self.unauthorized(environ,
+                                     start_response,
+                                     str(e),
+                                     'krbprincipal-expired')
+        except UserLocked as e:
+            return self.unauthorized(environ,
+                                     start_response,
+                                     str(e),
+                                     'user-locked')
 
         return self.finalize_kerberos_acquisition('login_password', ipa_ccache_name, environ, start_response)
 
@@ -984,9 +1025,18 @@ class login_password(Backend, KerberosSession, HTTP_Status):
             if ('kinit: Cannot read password while '
                     'getting initial credentials') in str(e):
                 raise PasswordExpired(principal=principal, message=unicode(e))
+            elif ('kinit: Client\'s entry in database'
+                  ' has expired while getting initial credentials') in str(e):
+                raise KrbPrincipalExpired(principal=principal,
+                                          message=unicode(e))
+            elif ('kinit: Clients credentials have been revoked '
+                  'while getting initial credentials') in str(e):
+                raise UserLocked(principal=principal,
+                                 message=unicode(e))
             raise InvalidSessionPassword(principal=principal,
                                          message=unicode(e))
 
+
 class change_password(Backend, HTTP_Status):
 
     content_type = 'text/plain'
@@ -1190,6 +1240,7 @@ class xmlserver_session(xmlserver, KerberosSession):
         super(xmlserver_session, self).__init__(api)
         name = '{0}_{1}'.format(self.__class__.__name__, id(self))
         auth_mgr = AuthManagerKerb(name)
+        session_mgr = get_session_mgr()
         session_mgr.auth_mgr.register(auth_mgr.name, auth_mgr)
 
     def _on_finalize(self):
@@ -1213,6 +1264,7 @@ class xmlserver_session(xmlserver, KerberosSession):
         self.debug('WSGI xmlserver_session.__call__:')
 
         # Load the session data
+        session_mgr = get_session_mgr()
         session_data = session_mgr.load_session_data(environ.get('HTTP_COOKIE'))
         session_id = session_data['session_id']
 
