@@ -368,6 +368,10 @@ class BaseTestLegacyClient(object):
         cls.master.run_command(['ipa', 'user-del', 'disabledipauser'],
                                 raiseonerr=False)
 
+        # Remove information about trust from AD, if domain was defined
+        if hasattr(cls, 'ad_domain'):
+            tasks.remove_trust_info_from_ad(cls.master, cls.ad_domain)
+
         # Also unapply fixes on the legacy client, if defined
         if hasattr(cls, 'legacy_client'):
             tasks.unapply_fixes(cls.legacy_client)
