@@ -24,7 +24,6 @@ Test the `ipalib.frontend` module.
 # FIXME: Pylint errors
 # pylint: disable=no-member
 import pytest
-
 import six
 
 from ipatests.util import raises, read_only
@@ -283,11 +282,11 @@ class test_Command(ClassChecker):
                 return False
         o = self.cls(api)
         o.finalize()
-        assert type(o.args) is plugable.NameSpace
+        assert type(o.args) is NameSpace
         assert len(o.args) == 0
         args = ('destination', 'source?')
         ns = self.get_instance(args=args).args
-        assert type(ns) is plugable.NameSpace
+        assert type(ns) is NameSpace
         assert len(ns) == len(args)
         assert list(ns) == ['destination', 'source']
         assert type(ns.destination) is parameters.Str
@@ -340,11 +339,11 @@ class test_Command(ClassChecker):
                 return False
         o = self.cls(api)
         o.finalize()
-        assert type(o.options) is plugable.NameSpace
+        assert type(o.options) is NameSpace
         assert len(o.options) == 1
         options = ('target', 'files*')
         ns = self.get_instance(options=options).options
-        assert type(ns) is plugable.NameSpace
+        assert type(ns) is NameSpace
         assert len(ns) == len(options) + 1
         assert list(ns) == ['target', 'files', 'version']
         assert type(ns.target) is parameters.Str
@@ -364,7 +363,7 @@ class test_Command(ClassChecker):
                 return False
         inst = self.cls(api)
         inst.finalize()
-        assert type(inst.output) is plugable.NameSpace
+        assert type(inst.output) is NameSpace
         assert list(inst.output) == ['result']
         assert type(inst.output.result) is output.Output
 
@@ -402,35 +401,6 @@ class test_Command(ClassChecker):
         for o in items:
             assert type(o) is output.Output
 
-    def test_soft_validate(self):
-        """
-        Test the `ipalib.frontend.Command.soft_validate` method.
-        """
-        class api(object):
-            env = config.Env(context='cli')
-            @staticmethod
-            def is_production_mode():
-                return False
-        class user_add(frontend.Command):
-            takes_args = parameters.Str('uid',
-                normalizer=lambda value: value.lower(),
-                default_from=lambda givenname, sn: givenname[0] + sn,
-            )
-
-            takes_options = ('givenname', 'sn')
-
-        cmd = user_add(api)
-        cmd.finalize()
-        assert list(cmd.params) == ['givenname', 'sn', 'uid', 'version']
-        ret = cmd.soft_validate({})
-        assert sorted(ret['values']) == ['version']
-        assert sorted(ret['errors']) == ['givenname', 'sn', 'uid']
-        assert cmd.soft_validate(dict(givenname=u'First', sn=u'Last')) == dict(
-            values=dict(givenname=u'First', sn=u'Last', uid=u'flast',
-                        version=None),
-            errors=dict(),
-        )
-
     def test_convert(self):
         """
         Test the `ipalib.frontend.Command.convert` method.
@@ -491,11 +461,10 @@ class test_Command(ClassChecker):
         api.finalize()
         o = my_cmd(api)
         o.finalize()
-        e = o(**kw)  # pylint: disable=not-callable
+        e = o.get_default(**kw)  # pylint: disable=not-callable
         assert type(e) is dict
-        assert 'result' in e
-        assert 'option2' in e['result']
-        assert e['result']['option2'] == u'some value'
+        assert 'option2' in e
+        assert e['option2'] == u'some value'
 
     def test_validate(self):
         """
@@ -523,11 +492,8 @@ class test_Command(ClassChecker):
         fail = dict(okay)
         fail['option0'] = u'whatever'
         e = raises(errors.ValidationError, sub.validate, **fail)
-        assert_equal(e.name, 'option0')
-        assert_equal(e.value, u'whatever')
+        assert_equal(e.name, u'option0')
         assert_equal(e.error, u"must equal 'option0'")
-        assert e.rule.__class__.__name__ == 'Rule'
-        assert e.index is None
 
         # Check with a missing required arg
         fail = dict(okay)
@@ -577,7 +543,7 @@ class test_Command(ClassChecker):
         o = self.get_instance(args=('one', 'two'), options=('three', 'four'))
         e = raises(errors.OverlapError, o.args_options_2_params,
             1, 2, three=3, two=2, four=4, one=1)
-        assert e.names == ['one', 'two']
+        assert e.names == "['one', 'two']"
 
         # Test the permutations:
         o = self.get_instance(args=('one', 'two*'), options=('three', 'four'))
@@ -634,9 +600,9 @@ class test_Command(ClassChecker):
         Test the `ipalib.frontend.Command.params_2_args_options` method.
         """
         o = self.get_instance(args='one', options='two')
-        assert o.params_2_args_options() == ((None,), {})
+        assert o.params_2_args_options() == ((), {})
         assert o.params_2_args_options(one=1) == ((1,), {})
-        assert o.params_2_args_options(two=2) == ((None,), dict(two=2))
+        assert o.params_2_args_options(two=2) == ((), dict(two=2))
         assert o.params_2_args_options(two=2, one=1) == ((1,), dict(two=2))
 
     def test_run(self):
@@ -780,13 +746,13 @@ class test_Command(ClassChecker):
 
         wrong = dict(foo=17.9, bar=[18])
         e = raises(TypeError, inst.validate_output, wrong)
-        assert str(e) == '%s:\n  output[%r]: need %r; got %r: %r' % (
+        assert str(e) == '%s:\n  output[%r]: need (%r,); got %r: %r' % (
             'Complex.validate_output()', 'foo', int, float, 17.9
         )
 
         wrong = dict(foo=18, bar=17)
         e = raises(TypeError, inst.validate_output, wrong)
-        assert str(e) == '%s:\n  output[%r]: need %r; got %r: %r' % (
+        assert str(e) == '%s:\n  output[%r]: need (%r,); got %r: %r' % (
             'Complex.validate_output()', 'bar', list, int, 17
         )
 
@@ -843,16 +809,16 @@ class test_Command(ClassChecker):
                 'foo',
             )
             takes_options = (
-                Str('bar', flags='no_output'),
+                Str('bar'),
                 'baz',
             )
 
         inst = example(api)
         inst.finalize()
         assert list(inst.get_output_params()) == [
-            'one', 'two', 'three', inst.params.foo, inst.params.baz
+            'one', 'two', 'three'
         ]
-        assert list(inst.output_params) == ['one', 'two', 'three', 'foo', 'baz']
+        assert list(inst.output_params) == ['one', 'two', 'three']
 
 
 class test_LocalOrRemote(ClassChecker):
@@ -896,13 +862,13 @@ class test_LocalOrRemote(ClassChecker):
         api.finalize()
         cmd = api.Command.example
         assert cmd(version=u'2.47') == dict(
-            result=('execute', (None,), dict(version=u'2.47', server=False))
+            result=('execute', (), dict(version=u'2.47'))
         )
         assert cmd(u'var', version=u'2.47') == dict(
-            result=('execute', (u'var',), dict(version=u'2.47', server=False))
+            result=('execute', (u'var',), dict(version=u'2.47'))
         )
         assert cmd(server=True, version=u'2.47') == dict(
-            result=('forward', (None,), dict(version=u'2.47', server=True))
+            result=('forward', (), dict(version=u'2.47', server=True))
         )
         assert cmd(u'var', server=True, version=u'2.47') == dict(
             result=('forward', (u'var',), dict(version=u'2.47', server=True))
@@ -914,13 +880,13 @@ class test_LocalOrRemote(ClassChecker):
         api.finalize()
         cmd = api.Command.example
         assert cmd(version=u'2.47') == dict(
-            result=('execute', (None,), dict(version=u'2.47', server=False))
+            result=('execute', (), dict(version=u'2.47', server=False))
         )
         assert cmd(u'var', version=u'2.47') == dict(
             result=('execute', (u'var',), dict(version=u'2.47', server=False))
         )
         assert cmd(server=True, version=u'2.47') == dict(
-            result=('execute', (None,), dict(version=u'2.47', server=True))
+            result=('execute', (), dict(version=u'2.47', server=True))
         )
         assert cmd(u'var', server=True, version=u'2.47') == dict(
             result=('execute', (u'var',), dict(version=u'2.47', server=True))
@@ -956,6 +922,9 @@ class test_Object(ClassChecker):
                     self.name = '%s_%s' % (obj_name, attr_name)
                 else:
                     self.name = name
+                self.bases = (DummyAttribute,)
+                self.version = '1'
+                self.full_name = '{}/{}'.format(self.name, self.version)
                 self.param = frontend.create_param(attr_name)
 
             def __clone__(self, attr_name):
@@ -974,15 +943,18 @@ class test_Object(ClassChecker):
         methods_format = 'method_%d'
 
         class FakeAPI(object):
-            Method = plugable.NameSpace(
-                get_attributes(cnt, methods_format)
-            )
+            def __init__(self):
+                self._API__plugins = get_attributes(cnt, methods_format)
+                self._API__default_map = {}
+                self.Method = plugable.APINameSpace(self, DummyAttribute)
             def __contains__(self, key):
                 return hasattr(self, key)
             def __getitem__(self, key):
                 return getattr(self, key)
             def is_production_mode(self):
                 return False
+            def _get(self, plugin):
+                return plugin
         api = FakeAPI()
         assert len(api.Method) == cnt * 3
 
@@ -994,7 +966,7 @@ class test_Object(ClassChecker):
         assert read_only(o, 'api') is api
 
         namespace = o.methods
-        assert isinstance(namespace, plugable.NameSpace)
+        assert isinstance(namespace, NameSpace)
         assert len(namespace) == cnt
         f = methods_format
         for i in range(cnt):
@@ -1009,13 +981,13 @@ class test_Object(ClassChecker):
         # Test params instance attribute
         o = self.cls(api)
         ns = o.params
-        assert type(ns) is plugable.NameSpace
+        assert type(ns) is NameSpace
         assert len(ns) == 0
         class example(self.cls):
             takes_params = ('banana', 'apple')
         o = example(api)
         ns = o.params
-        assert type(ns) is plugable.NameSpace
+        assert type(ns) is NameSpace
         assert len(ns) == 2, repr(ns)
         assert list(ns) == ['banana', 'apple']
         for p in ns():
@@ -1053,7 +1025,7 @@ class test_Object(ClassChecker):
         assert pk.name == 'three'
         assert pk.primary_key is True
         assert o.params[2] is o.primary_key
-        assert isinstance(o.params_minus_pk, plugable.NameSpace)
+        assert isinstance(o.params_minus_pk, NameSpace)
         assert list(o.params_minus_pk) == ['one', 'two', 'four']
 
         # Test with multiple primary_key:
@@ -1140,7 +1112,7 @@ class test_Attribute(ClassChecker):
         """
         user_obj = 'The user frontend.Object instance'
         class api(object):
-            Object = dict(user=user_obj)
+            Object = {("user", "1"): user_obj}
             @staticmethod
             def is_production_mode():
                 return False
