@@ -209,14 +209,6 @@ class test_xmlclient(PluginTester):
         class user_add(Command):
             pass
 
-        # Test that ValueError is raised when forwarding a command that is not
-        # in api.Command:
-        (o, api, home) = self.instance('Backend', in_server=False)
-        e = raises(ValueError, o.forward, 'user_add')
-        assert str(e) == '%s.forward(): %r not in api.Command' % (
-            'xmlclient', 'user_add'
-        )
-
         (o, api, home) = self.instance('Backend', user_add, in_server=False)
         args = (binary_bytes, utf8_bytes, unicode_str)
         kw = dict(one=binary_bytes, two=utf8_bytes, three=unicode_str)
@@ -267,7 +259,7 @@ class test_xml_introspection(object):
     @classmethod
     def setup_class(self):
         try:
-            api.Backend.xmlclient.connect(fallback=False)
+            api.Backend.xmlclient.connect()
         except (errors.NetworkError, IOError):
             raise nose.SkipTest('%r: Server not available: %r' %
                                 (__name__, api.env.xmlrpc_uri))
