@@ -35,11 +35,8 @@ import nss.nss as nss
 import six
 
 from ipaserver.plugins.ldap2 import ldap2
-from ipalib.plugins.service import service, service_show
-from ipalib.plugins.host import host
 from ipalib import api, x509, create_api, errors
 from ipapython import ipautil
-from ipaplatform.paths import paths
 from ipapython.dn import DN
 
 if six.PY3:
@@ -55,7 +52,6 @@ class test_ldap(object):
     def setup(self):
         self.conn = None
         self.ldapuri = 'ldap://%s' % ipautil.format_netloc(api.env.host)
-        self.ccache = paths.TMP_KRB5CC % os.getuid()
         nss.nss_init_nodb()
         self.dn = DN(('krbprincipalname','ldap/%s@%s' % (api.env.host, api.env.realm)),
                      ('cn','services'),('cn','accounts'),api.env.basedn)
@@ -79,10 +75,8 @@ class test_ldap(object):
         """
         Test a GSSAPI LDAP bind using ldap2
         """
-        if not ipautil.file_exists(self.ccache):
-            raise nose.SkipTest('Missing ccache %s' % self.ccache)
         self.conn = ldap2(api, ldap_uri=self.ldapuri)
-        self.conn.connect(ccache='FILE:%s' % self.ccache)
+        self.conn.connect()
         entry_attrs = self.conn.get_entry(self.dn, ['usercertificate'])
         cert = entry_attrs.get('usercertificate')
         cert = cert[0]
@@ -117,11 +111,7 @@ class test_ldap(object):
         # a client-only api. Then we register in the commands and objects
         # we need for the test.
         myapi = create_api(mode=None)
-        myapi.bootstrap(context='cli', in_server=True, in_tree=True)
-        myapi.add_plugin(ldap2)
-        myapi.add_plugin(host)
-        myapi.add_plugin(service)
-        myapi.add_plugin(service_show)
+        myapi.bootstrap(context='cli', in_server=True)
         myapi.finalize()
 
         pwfile = api.env.dot_ipa + os.sep + ".dmpw"
@@ -230,7 +220,7 @@ class test_LDAPEntry(object):
     def test_popitem(self):
         e = self.entry
         assert e.popitem() == ('cn', self.cn1)
-        list(e) == []
+        assert list(e) == []
 
     def test_setdefault(self):
         e = self.entry
