@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-Test the `ipalib/plugins/hbacrule.py` module.
+Test the `ipaserver/plugins/hbacrule.py` module.
 """
 
 from nose.tools import raises, assert_raises  # pylint: disable=E0611
