@@ -3,14 +3,9 @@
 #
 
 """
-Test the `ipalib/plugins/stageuser.py` module.
+Test the `ipaserver/plugins/stageuser.py` module.
 """
 
-
-import datetime
-import ldap
-import re
-import functools
 import pytest
 
 import six
@@ -18,14 +13,8 @@ import six
 from collections import OrderedDict
 from ipalib import api, errors
 
-from ipatests.test_xmlrpc import objectclasses
-from ipatests.test_xmlrpc.xmlrpc_test import (
-    XMLRPC_test, fuzzy_digits, fuzzy_uuid, fuzzy_password, fuzzy_string,
-    fuzzy_dergeneralizedtime, add_sid, add_oc, raises_exact)
+from ipatests.test_xmlrpc.xmlrpc_test import XMLRPC_test, raises_exact
 
-from ipatests.util import (
-    assert_equal, assert_deepequal, assert_not_equal, raises, get_user_dn)
-from ipapython.dn import DN
 from ipatests.test_xmlrpc.tracker.user_plugin import UserTracker
 from ipatests.test_xmlrpc.tracker.group_plugin import GroupTracker
 from ipatests.test_xmlrpc.tracker.stageuser_plugin import StageUserTracker
@@ -356,9 +345,9 @@ class TestCreateInvalidAttributes(XMLRPC_test):
         stageduser.ensure_missing()
         command = stageduser.make_create_command(
             options={u'krbprincipalname': invalidrealm2})
-        with raises_exact(errors.MalformedUserPrincipal(
-                message=u'Principal is not of the form user@REALM: \'%s\'' %
-                invalidrealm2)):
+        with raises_exact(errors.ConversionError(
+                name='principal', error="Malformed principal: '{}'".format(
+                    invalidrealm2))):
             command()
 
 
@@ -647,9 +636,7 @@ class TestGroups(XMLRPC_test):
     def test_remove_preserved_from_group(self, user, group):
         user.ensure_exists()
         group.ensure_exists()
-        command = group.make_add_member_command(options={u'user': user.uid})
-        result = command()
-        group.check_add_member(result)
+        command = group.add_member(options={u'user': user.uid})
 
         command = group.make_retrieve_command()
         result = command()
