@@ -12,12 +12,13 @@ from ipatests.test_xmlrpc import objectclasses
 from ipatests.test_xmlrpc.xmlrpc_test import (
     fuzzy_digits, fuzzy_uuid, raises_exact)
 from ipatests.test_xmlrpc.tracker.base import Tracker
+from ipatests.test_xmlrpc.tracker.kerberos_aliases import KerberosAliasMixin
 
 if six.PY3:
     unicode = str
 
 
-class UserTracker(Tracker):
+class UserTracker(KerberosAliasMixin, Tracker):
     """ Class for host plugin like tests """
 
     retrieve_keys = {
@@ -28,6 +29,7 @@ class UserTracker(Tracker):
         u'ipatokenradiusconfiglink', u'ipatokenradiususername',
         u'krbprincipalexpiration', u'usercertificate;binary',
         u'has_keytab', u'has_password', u'memberof_group', u'sshpubkeyfp',
+        u'krbcanonicalname', 'krbprincipalname'
     }
 
     retrieve_all_keys = retrieve_keys | {
@@ -36,8 +38,7 @@ class UserTracker(Tracker):
         u'l', u'mobile', u'krbextradata', u'krblastpwdchange',
         u'krbpasswordexpiration', u'pager', u'st', u'manager', u'cn',
         u'ipauniqueid', u'objectclass', u'mepmanagedentry',
-        u'displayname', u'gecos', u'initials', u'krbprincipalname',
-        u'preserved'}
+        u'displayname', u'gecos', u'initials', u'preserved'}
 
     retrieve_preserved_keys = (retrieve_keys - {u'memberof_group'}) | {
         u'preserved'}
@@ -51,8 +52,13 @@ class UserTracker(Tracker):
     update_keys = retrieve_keys - {u'dn'}
     activate_keys = retrieve_keys
 
-    find_keys = retrieve_keys - {u'mepmanagedentry', u'memberof_group'}
-    find_all_keys = retrieve_all_keys
+    find_keys = retrieve_keys - {
+        u'mepmanagedentry', u'memberof_group', u'has_keytab', u'has_password',
+        u'manager',
+    }
+    find_all_keys = retrieve_all_keys - {
+        u'has_keytab', u'has_password'
+    }
 
     primary_keys = {u'uid', u'dn'}
 
@@ -74,11 +80,21 @@ class UserTracker(Tracker):
             )
 
     def make_delete_command(self, no_preserve=True, preserve=False):
-        """ Make function that deletes a user using user-del """
+        """ Make function that deletes a user using user-del
+
+        Arguments 'preserve' and 'no_preserve' represent implemented
+        options --preserve and --no-preserve of user-del command,
+        which are mutually exclusive.
+        If --preserve=True and --no-preserve=False, the user is moved
+        to deleted container.
+        If --preserve=True and --no-preserve=True, an error is raised.
+        If --preserve=False and --no-preserver=True, user is deleted.
+        """
 
         if preserve and not no_preserve:
-            # necessary to change some user attributes due to moving
-            # to different container
+            # --preserve=True and --no-preserve=False - user is moved to
+            # another container, hence it is necessary to change some user
+            # attributes
             self.attrs[u'dn'] = DN(
                 ('uid', self.uid),
                 api.env.container_deleteuser,
@@ -141,6 +157,7 @@ class UserTracker(Tracker):
             uidnumber=[fuzzy_digits],
             gidnumber=[fuzzy_digits],
             krbprincipalname=[u'%s@%s' % (self.uid, self.api.env.realm)],
+            krbcanonicalname=[u'%s@%s' % (self.uid, self.api.env.realm)],
             mail=[u'%s@%s' % (self.uid, self.api.env.domain)],
             gecos=[u'%s %s' % (self.givenname, self.sn)],
             loginshell=[u'/bin/sh'],
@@ -178,7 +195,7 @@ class UserTracker(Tracker):
         Overriding Tracker method for setting self.attrs correctly;
          * most attributes stores its value in list
          * the rest can be overridden by expected_updates
-         * allow deleting parametrs if update value is None
+         * allow deleting parameters if update value is None
         """
         if expected_updates is None:
             expected_updates = {}
@@ -190,6 +207,12 @@ class UserTracker(Tracker):
         for key, value in updates.items():
             if value is None or value is '' or value is u'':
                 del self.attrs[key]
+            elif key == 'rename':
+                new_principal = u'{0}@{1}'.format(value, self.api.env.realm)
+                self.attrs['uid'] = [value]
+                self.attrs['krbcanonicalname'] = [new_principal]
+                if new_principal not in self.attrs['krbprincipalname']:
+                    self.attrs['krbprincipalname'].append(new_principal)
             else:
                 if type(value) is list:
                     self.attrs[key] = value
@@ -206,6 +229,9 @@ class UserTracker(Tracker):
             extra_keys=set(updates.keys()) | set(expected_updates.keys())
         )
 
+        if 'rename' in updates:
+            self.uid = self.attrs['uid'][0]
+
     def check_create(self, result, extra_keys=()):
         """ Check 'user-add' command result """
         expected = self.filter_attrs(self.create_keys | set(extra_keys))
@@ -477,3 +503,10 @@ class UserTracker(Tracker):
                 'description': [u'Account administrators group'],
             },
         ), result)
+
+    #  Kerberos aliases methods
+    def _make_add_alias_cmd(self):
+        return self.make_command('user_add_principal', self.name)
+
+    def _make_remove_alias_cmd(self):
+        return self.make_command('user_remove_principal', self.name)
