@@ -28,7 +28,9 @@ import tempfile
 import shutil
 import re
 import uuid
+import pytest
 from contextlib import contextmanager
+from pprint import pformat
 
 import six
 import ldap
@@ -40,7 +42,9 @@ from ipalib import api
 from ipalib.plugable import Plugin
 from ipalib.request import context
 from ipapython.dn import DN
-from ipapython.ipautil import private_ccache, kinit_password, run
+from ipapython.ipautil import (
+    private_ccache, kinit_password, kinit_keytab, run
+)
 from ipaplatform.paths import paths
 
 if six.PY3:
@@ -205,7 +209,7 @@ class Fuzzy(object):
     >>> fuzzy.test  # doctest:+ELLIPSIS
     <function <lambda> at 0x...>
 
-    To aid debugging, `Fuzzy.__repr__()` revealse these kwargs as well:
+    To aid debugging, `Fuzzy.__repr__()` reveals these kwargs as well:
 
     >>> fuzzy  # doctest:+ELLIPSIS
     Fuzzy('.+', <type 'str'>, <function <lambda> at 0x...>)
@@ -273,18 +277,29 @@ LEN = """assert_deepequal: list length mismatch.
   %s
   len(expected) = %r
   len(got) = %r
-  expected = %r
-  got = %r
+  expected = %s
+  got = %s
   path = %r"""
 
 KEYS = """assert_deepequal: dict keys mismatch.
   %s
   missing keys = %r
   extra keys = %r
-  expected = %r
-  got = %r
+  expected = %s
+  got = %s
   path = %r"""
 
+EXPECTED_LEN = len('  expected = ')
+GOT_LEN = len('  got = ')
+
+
+def struct_to_string(struct, indent=1):
+    """
+    Function to pretty-format a structure and optionally indent its lines
+    so they match the visual indention of the first line
+    """
+    return pformat(struct).replace('\n', '\n' + ' ' * indent)
+
 
 def assert_deepequal(expected, got, doc='', stack=tuple()):
     """
@@ -315,6 +330,13 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
     Note that lists and tuples are considered equivalent, and the order of
     their elements does not matter.
     """
+    if pytest.config.getoption("pretty_print"):  # pylint: disable=no-member
+        expected_str = struct_to_string(expected, EXPECTED_LEN)
+        got_str = struct_to_string(got, GOT_LEN)
+    else:
+        expected_str = repr(expected)
+        got_str = repr(got)
+
     if isinstance(expected, tuple):
         expected = list(expected)
     if isinstance(got, tuple):
@@ -329,7 +351,8 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
     if isinstance(expected, (list, tuple)):
         if len(expected) != len(got):
             raise AssertionError(
-                LEN % (doc, len(expected), len(got), expected, got, stack)
+                LEN % (doc, len(expected), len(got), expected_str, got_str,
+                       stack)
             )
         # Sort list elements, unless they are dictionaries
         if expected and isinstance(expected[0], dict):
@@ -352,8 +375,8 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
         extra = set(got).difference(expected)
         if missing or extra:
             raise AssertionError(KEYS % (
-                    doc, sorted(missing), sorted(extra), expected, got, stack
-                )
+                    doc, sorted(missing), sorted(extra), expected_str, got_str,
+                    stack)
             )
         for key in sorted(expected):
             e_sub = expected[key]
@@ -652,6 +675,9 @@ class MockLDAP(object):
         except ldap.ALREADY_EXISTS:
             pass
 
+    def mod_entry(self, dn, mods):
+        self.connection.modify_s(dn, mods)
+
     def del_entry(self, dn):
         try:
             self.connection.delete_s(dn)
@@ -662,6 +688,13 @@ class MockLDAP(object):
         if self.connection is not None:
             self.connection.unbind_s()
 
+    # contextmanager protocol
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.unbind()
+
 
 def prepare_config(template, values):
     with open(template) as f:
@@ -683,7 +716,28 @@ def unlock_principal_password(user, oldpw, newpw):
 
 
 @contextmanager
-def change_principal(user, password, client=None, path=None):
+def change_principal(principal, password=None, client=None, path=None,
+                     canonicalize=False, enterprise=False, keytab=None):
+    """Temporarily change the kerberos principal
+
+    Most of the test cases run with the admin ipa user which is granted
+    all access and exceptions from rules on some occasions.
+
+    When the test needs to test for an application of some kind
+    of a restriction it needs to authenticate as a different principal
+    with required set of rights to the operation.
+
+    The context manager changes the principal identity in two ways:
+
+    * using password
+    * using keytab
+
+    If the context manager is to be used with a keytab, the keytab
+    option must be its absolute path.
+
+    The context manager can be used to authenticate with enterprise
+    principals and aliases when given respective options.
+    """
 
     if path:
         ccache_name = path
@@ -696,16 +750,58 @@ def change_principal(user, password, client=None, path=None):
 
     client.Backend.rpcclient.disconnect()
 
-    with private_ccache(ccache_name):
-        kinit_password(user, password, ccache_name)
+    try:
+        with private_ccache(ccache_name):
+            if keytab:
+                kinit_keytab(principal, keytab, ccache_name)
+            else:
+                kinit_password(principal, password, ccache_name,
+                               canonicalize=canonicalize,
+                               enterprise=enterprise)
+            client.Backend.rpcclient.connect()
+
+            try:
+                yield
+            finally:
+                client.Backend.rpcclient.disconnect()
+    finally:
         client.Backend.rpcclient.connect()
 
-        try:
-            yield
-        finally:
-            client.Backend.rpcclient.disconnect()
 
-    client.Backend.rpcclient.connect()
+@contextmanager
+def get_entity_keytab(principal, options=None):
+    """Requests a keytab for an entity
+
+    The keytab will generate new keys if not specified
+    otherwise in the options.
+    To retrieve existing keytab, use the -r option
+    """
+    keytab_filename = os.path.join('/tmp', str(uuid.uuid4()))
+
+    try:
+        cmd = [paths.IPA_GETKEYTAB, '-p', principal, '-k', keytab_filename]
+
+        if options:
+            cmd.extend(options)
+        run(cmd)
+
+        yield keytab_filename
+    finally:
+        os.remove(keytab_filename)
+
+
+@contextmanager
+def host_keytab(hostname, options=None):
+    """Retrieves keytab for a particular host
+
+    After leaving the context manager, the keytab file is
+    deleted.
+    """
+    principal = u'host/{}'.format(hostname)
+
+    with get_entity_keytab(principal, options) as keytab:
+        yield keytab
+
 
 def get_group_dn(cn):
     return DN(('cn', cn), api.env.container_group, api.env.basedn)
